// ScrnWin.c for the ScrnXtra Xtra for Director 5 and later
// Based on LPUtils code written by Kent Kersten from samples by Macromedia
// April 1996 for Director 5.0
// Version 2 code in February 1997

// ********   NEED MUCH MORE ERROR CHECKING
// especially in the bitmap routines


#ifdef	_MSC_VER

#include	<errno.h>

#include "beepimp.h"


#define UNUSED(x) 			x

#define	WIDTHBYTES(bits)	(((bits) + 31) / 32 * 4)
#define	ERR_GETDC			8
#define PALVERSION			0x300
#define	BYTES_PER_READ		32767
#define	DIB_HEADER_MARKER	((WORD) ('M' << 8) | 'B')

#define PROPBUF_SIZE 		256
#define PROPNAME_SIZE		32

void GetDirRGB(int, int *, int *, int *);



//---------------------------------------------------------------------
//
// Function:   CopyHandle
//
// Purpose:    Makes a copy of the given global memory block.  Returns
//             a handle to the new memory block (NULL on error).
//
//             Routine stolen verbatim out of ShowDIB.
//
// Parms:      h == Handle to global memory to duplicate.
//
// Returns:    Handle to new global memory block.
//
// History:   Date      Reason
//             ???      Created
//
//---------------------------------------------------------------------

HANDLE CopyHandle(HANDLE h)
{
#if (_MSC_VER <= 800)
   BYTE huge *lpCopy;
   BYTE huge *lp;
#else
   BYTE *lpCopy;
   BYTE *lp;
#endif
   HANDLE     hCopy;
   DWORD      dwLen;

   if (!h)
      return NULL;

   dwLen = GlobalSize (h);

   if (hCopy = GlobalAlloc (GHND, dwLen))
      {
#if (_MSC_VER <= 800)
      lpCopy = (BYTE huge *)GlobalLock (hCopy);
      lp     = (BYTE huge *)GlobalLock (h);
#else
      lpCopy = (BYTE *)GlobalLock (hCopy);
      lp     = (BYTE *)GlobalLock (h);
#endif
      while (dwLen--)
         *lpCopy++ = *lp++;

      GlobalUnlock (hCopy);
      GlobalUnlock (h);
      }

   return hCopy;
}

		
static WORD DIBNumColors(LPSTR lpbi)
{
	WORD wBitCount;
	DWORD dwClrUsed;

	// If this is a Windows style DIB, the number of colors in the
	//  color table can be less than the number of bits per pixel
	//  allows for (i.e. lpbi->biClrUsed can be set to some value).
	//  If this is the case, return the appropriate value.
	dwClrUsed = ((LPBITMAPINFOHEADER) lpbi)->biClrUsed;

	if(dwClrUsed)
		return((WORD) dwClrUsed);

	// Calculate the number of colors in the color table based on
	//  the number of bits per pixel for the DIB.
	wBitCount = ((LPBITMAPINFOHEADER) lpbi)->biBitCount;

	switch(wBitCount) {
		case 1:
			return(2);
		case 4:
			return(16);
		case 8:
			return(256);
		default:
			return(0);
	}
}

			  
static WORD PaletteSize(LPSTR lpbi)
{
	return(DIBNumColors(lpbi) * sizeof(RGBQUAD));
}

				 
static LPSTR FindDIBBits(LPSTR lpbi)
{
	return(lpbi + *(LPDWORD) lpbi + PaletteSize(lpbi));
}


static void InitBitmapInfoHeader(LPBITMAPINFOHEADER lpBmInfoHdr,
								DWORD dwWidth,
								DWORD dwHeight,
								int nBPP)
{
	memset(lpBmInfoHdr, 0, sizeof(BITMAPINFOHEADER));

	lpBmInfoHdr->biSize      = sizeof (BITMAPINFOHEADER);
	lpBmInfoHdr->biWidth     = dwWidth;
	lpBmInfoHdr->biHeight    = dwHeight;
	lpBmInfoHdr->biPlanes    = 1;
	lpBmInfoHdr->biClrImportant = 0;	// signify that all of the device colors are important

	if(nBPP <= 1)
		nBPP = 1;
	else if(nBPP <= 4)
		nBPP = 4;
	else if(nBPP <= 8)
		nBPP = 8;
	else
		nBPP = 24;

	lpBmInfoHdr->biBitCount  = nBPP;
	lpBmInfoHdr->biSizeImage = WIDTHBYTES(dwWidth * nBPP) * dwHeight;
}

	   
static HANDLE BitmapToDIB(HBITMAP hBitmap, HPALETTE hPal)
{
	BITMAP             Bitmap;
	BITMAPINFOHEADER   bmInfoHdr;
	LPBITMAPINFOHEADER lpbmInfoHdr;
	LPSTR              lpBits, lpPal;
	HDC                hMemDC;
	HANDLE             hDIB;
	HPALETTE           hOldPal;

	// Do some setup -- make sure the Bitmap passed in is valid,
	//  get info on the bitmap (like its height, width, etc.),
	//  then setup a BITMAPINFOHEADER.
	hOldPal = (HANDLE) NULL;

	if(!hBitmap)
		return((HANDLE) NULL);

	if(!GetObject(hBitmap, sizeof(Bitmap), (LPSTR) &Bitmap))
		return((HANDLE) NULL);

	InitBitmapInfoHeader(&bmInfoHdr, 
						Bitmap.bmWidth, 
						Bitmap.bmHeight, 
						Bitmap.bmPlanes * Bitmap.bmBitsPixel);

	// Now allocate memory for the DIB.  Then, set the BITMAPINFOHEADER
	//  into this memory, and find out where the bitmap bits go.
	hDIB = GlobalAlloc(GHND, sizeof(BITMAPINFOHEADER) + 
				PaletteSize((LPSTR) &bmInfoHdr) + bmInfoHdr.biSizeImage);

	if(!hDIB)
		return((HANDLE) NULL);

	lpbmInfoHdr  = (LPBITMAPINFOHEADER) GlobalLock(hDIB);
	*lpbmInfoHdr = bmInfoHdr;
	lpBits       = FindDIBBits((LPSTR) lpbmInfoHdr);

	// Now, we need a DC to hold our bitmap.  If the app passed us
	//  a palette, it should be selected into the DC.
	hMemDC = GetDC((HWND) NULL);

	if(hPal) {
		hOldPal = SelectPalette(hMemDC, hPal, FALSE);
		RealizePalette(hMemDC);
	}

	// We're finally ready to get the DIB.  Call the driver and let
	//  it work on our bitmap.  It will fill in the color table,
	//  and bitmap bits of our global memory block.
	if(!GetDIBits(hMemDC, hBitmap, 0, Bitmap.bmHeight, lpBits, 
				(LPBITMAPINFO) lpbmInfoHdr, DIB_RGB_COLORS)) {
		GlobalUnlock(hDIB);
		GlobalFree(hDIB);
		hDIB = (HANDLE) NULL;
	}
	else
		GlobalUnlock(hDIB);

	// Finally, clean up and return.
	if(hOldPal)
		SelectPalette(hMemDC, hOldPal, FALSE);

	ReleaseDC((HWND) NULL, hMemDC);

	return(hDIB);
}

	   
// Find out the number of palette entries on this device.

static int PalEntriesOnDevice(HDC hDC)
{
	int nColors;

	nColors = GetDeviceCaps(hDC, SIZEPALETTE);

	// For non-palette devices, we'll use the # of system colors
	// for our palette size.
	if(!nColors)
		nColors = GetDeviceCaps(hDC, NUMCOLORS);

	return(nColors);
}

				 
static HPALETTE GetSystemPalette(HDC stageDC)
{
	int           i, nColors, r,g,b;
	HDC           hDC, sourceDC;
	HPALETTE      hPal;
	HANDLE        hLogPal, oldPal;
	LPLOGPALETTE  lpLogPal;

	nColors = PalEntriesOnDevice(stageDC /*hDC*/);

//	sourceDC = GetDC(GetDesktopWindow());

	if(nColors > 256) {
		hPal = SelectPalette(stageDC, GetStockObject(DEFAULT_PALETTE), FALSE);
		SelectPalette(stageDC, hPal, FALSE);
		return(hPal);
	}


 	hPal = (HPALETTE) NULL;

 
	// Find out how many palette entries we want.
//	hDC = GetDC((HWND) NULL);
//	if(!hDC)
//		return((HPALETTE) NULL);

//	ReleaseDC((HWND) NULL, hDC);

	// Allocate room for the palette and lock it.
	hLogPal = GlobalAlloc(GHND, sizeof(LOGPALETTE) + nColors * sizeof(PALETTEENTRY));
	if(!hLogPal)
		return((HPALETTE) NULL);

	lpLogPal = (LPLOGPALETTE) GlobalLock(hLogPal);

	lpLogPal->palVersion    = PALVERSION;
	lpLogPal->palNumEntries = nColors;

	for(i = 0;i < nColors;i++) {
		GetDirRGB(i, &r, &g, &b);
		lpLogPal->palPalEntry[i].peRed = r;
		lpLogPal->palPalEntry[i].peGreen = g;
		lpLogPal->palPalEntry[i].peBlue = b;
		lpLogPal->palPalEntry[i].peFlags = 0;
	}

	// Go ahead and create the palette.  Once it's created,
	//  we no longer need the LOGPALETTE, so free it.
	hPal = CreatePalette(lpLogPal);

	GlobalUnlock(hLogPal);
	GlobalFree(hLogPal);

//	ReleaseDC(GetDesktopWindow(), sourceDC);

	return(hPal);
}


// Write the bitmap out to a disk file.

int WriteDIB(LPSTR szFile, HANDLE hdib)
{
	int                 fh;
	BITMAPFILEHEADER	hdr;
	LPBITMAPINFOHEADER  lpbi;
	OFSTRUCT            of;

//MessageBox(NULL, "WriteDIB #1...", "debug message", MB_OK);

	if(!hdib)
		return(FALSE);

	if((fh = OpenFile(szFile, &of, OF_CREATE | OF_READWRITE)) == -1)
		return(-10);

//MessageBox(NULL, "WriteDIB #2...", "debug message", MB_OK);

    lpbi = (VOID FAR *) GlobalLock(hdib);

    /* Fill in the fields of the file header */
    hdr.bfType		= DIB_HEADER_MARKER;
    hdr.bfSize		= GlobalSize(hdib) + sizeof(BITMAPFILEHEADER);
    hdr.bfReserved1 = 0;
    hdr.bfReserved2 = 0;
    hdr.bfOffBits   = (DWORD) sizeof(BITMAPFILEHEADER) + lpbi->biSize +
                          PaletteSize((LPSTR)lpbi);

//MessageBox(NULL, "WriteDIB #3...", "debug message", MB_OK);

    /* Write the file header */
	if(!_hwrite(fh, (LPSTR) &hdr, sizeof(BITMAPFILEHEADER))) {
		GlobalUnlock(hdib);
		_lclose(fh);
		return(-11);
	}

//MessageBox(NULL, "WriteDIB #4...", "debug message", MB_OK);

    /* Write the DIB header and the bits */
	if(!_hwrite(fh, (LPSTR) lpbi, GlobalSize(hdib))) {
		GlobalUnlock(hdib);
		_lclose(fh);
		return(-11);
	}

//MessageBox(NULL, "WriteDIB #5...", "debug message", MB_OK);

	GlobalUnlock(hdib);
	_lclose(fh);
	return(0);
}


// Return 0 for success, a negative error code otherwise.

int	GrabRect(HDC passedSrcDC, long rleft, long rtop, long rright, long rbottom,
				long newWidth, long newHeight,
				HBITMAP *ptrHBitmap, HANDLE *ptrHDIB, HPALETTE *ptrHPal, HDC stageDC)
{
	long destWidth, destHeight;
	HDC sourceDC, hMemoryDC;
	HBITMAP hBitmap, hOldBitmap;
	HPALETTE hPal;
	HANDLE hDIB;

hPal = SelectPalette(stageDC, GetStockObject(DEFAULT_PALETTE), FALSE);
SelectPalette(stageDC, hPal, FALSE);

	if(passedSrcDC)
		sourceDC = passedSrcDC;
	else {
		if((sourceDC = GetDC(GetDesktopWindow())) == (HDC) NULL)
			return(-201);
	}

	if((newWidth > 0) && (newHeight > 0)) {	// resize resulting picture
		destWidth = newWidth;
		destHeight = newHeight;
	}
	else {
		destWidth = rright - rleft;
		destHeight = rbottom - rtop;
	}

	if((hMemoryDC = CreateCompatibleDC(stageDC /*sourceDC*/)) == (HDC) NULL) {
		if(!passedSrcDC)
			ReleaseDC(GetDesktopWindow(), sourceDC);
		return(-202);
	}

//SelectPalette(hMemoryDC, hPal, FALSE);
//RealizePalette(hPal);

	if((hBitmap = CreateCompatibleBitmap(stageDC /*sourceDC*/,
				(int) destWidth, (int) destHeight)) == (HBITMAP) NULL) {
		DeleteDC(hMemoryDC);
		if(!passedSrcDC)
			ReleaseDC(GetDesktopWindow(), sourceDC);
		return(-203);
	}
	hOldBitmap = SelectObject(hMemoryDC, hBitmap);


//SelectPalette(hMemoryDC, hPal, FALSE);
//RealizePalette(hPal);


	StretchBlt(hMemoryDC, 0, 0, (int) destWidth, (int) destHeight,
			sourceDC, (int) rleft, (int) rtop, (int) (rright - rleft),
			(int) (rbottom - rtop), SRCCOPY);


SelectPalette(hMemoryDC, hPal, FALSE);
RealizePalette(hPal);


	SelectObject(hMemoryDC, hOldBitmap);
	DeleteDC(hMemoryDC);
	if(!passedSrcDC)
		ReleaseDC(GetDesktopWindow(), sourceDC);

//	hPal = GetSystemPalette(stageDC);
	hDIB = BitmapToDIB(hBitmap, hPal);

	*ptrHBitmap = hBitmap;			// pass these back to the caller
	*ptrHPal = hPal;
	*ptrHDIB = hDIB;

	return(0);
}


#endif		// _MSC_VER
