/*
	Copyright (c) 1998-2001 Macromedia, Inc.	All Rights Reserved

	You may utilize this source file to create and compile object code for use within products
	you may create. THIS CODE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, AND
	MACROMEDIA DISCLAIMS ALL IMPLIED WARRANTIES INCLUDING, BUT NOT LIMITED TO, MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.	IN NO EVENT WILL MACROMEDIA BE
	LIABLE TO YOU FOR ANY CONSEQUENTIAL, INDIRECT OR INCIDENTAL DAMAGES ARISING OUT OF YOUR
	USE OR INABILITY TO USE THIS CODE.
*/


#define INITGUID 1

#include "script.h"

#include <string.h>
#include <stdlib.h>
#include "xclassver.h"
#include "moatry.h"

#include "driservc.h"
#include "drivalue.h"

#include "mmivalue.h"
#include "mmillist.h"
#include "mmiplist.h"
#include "mmidate.h"
#include "mmiclr.h"

//-- PL--------------
#ifdef MACINTOSH
	#include <windows.h>
	#include <string.h>
	//#include <stdlib.h> // Malloc/Free
	#include <scrap.h>
#endif

#include "scrn.h"
//--/PL---------------

#define PROPBUF_SIZE	256
#define PROPNAME_SIZE	32

#define	SX_CAPTURE_STAGE	1			// sources for screen captures
#define	SX_CAPTURE_SCREEN	2

#define	SX_NORMAL_IMAGE		1			// tells whether to stretch the image or not
#define	SX_RESIZE_IMAGE		2

#define	SX_WRITE_MEMBER		1			// destinations for screen captures
#define	SX_WRITE_CLIPBOARD	2
#define	SX_WRITE_FILE		3


#include "scrn.h"
//-- PL dplac dans scrn.h
/***
// forward declarations for the internal functions that do all the dirty work
#ifdef MACINTOSH
	void ctopstr(char *);
	int WritePICT(char *, PicHandle);
	int	GrabRect(CGrafPtr, long, long, long, long, long, long, PicHandle *);
#else
	HANDLE CopyHandle(HANDLE);
	int WriteDIB(LPSTR, HANDLE);
	int	GrabRect(HDC, long, long, long, long, long, long, HBITMAP *, HANDLE *, HPALETTE *, HDC);
#endif
***/

//   Forward declarations for methods defined as functions here.

//static MoaError	XPly_CaptureMethod(ScrnXtra_IMoaMmXScript FAR *, PMoaDrCallInfo, long, long, long);

//ScrnXtra_IMoaMmXScript FAR *globalThis;//non utilis

/* Global to hold temporary argument values */
static	MoaMmValue	gTempArg;

// globales pour MyAlert()
PIMoaMmValue gMmValue;
PIMoaDrPlayer gDrPlayer;

/*******************************************************************************
 * SCRIPTING XTRA MESSAGE TABLE DESCRIPTION.
 *
 * The general format is:
 * xtra <nameOfXtra>
 * new object me [ args ... ]
 * <otherHandlerDefintions>
 * --
 * The first line must give the name of the Scripting xtra.
 * The remaining lines give the names of the handlers that this xtra implements
 * along with the required types of the arguments for each handler.
 * 
 * -- Pre-defined handler new 
 * The new handler will be called when a child object is created, 
 * the first argument is always the child object and you defined any remaining arguments.
 * The new handler is the place to initialize any memory required by the child object.
 * 
 * -- Simple Handler Definitions
 * Each handler definition line is format like this:
 * <handlerName> <argType1> <argName1>, <argType2> <argName2> ...
 * The first word is the handler name. Following this are types description for
 * the argument to the handler, each separated by a comma.
 * The argument name <argName>, may be omited.
 * Permited argument types are: 
 * 	integer 
 * 	float
 * 	string
 * 	symbol
 * 	object
 * 	any
 * 	*
 * For integer, float, string, symbol, and object, the type of the argument must 
 * match. The type any means allow any type. The asterisk (*) means any number and 
 * any type of arguments.
 * 
 * The first argument is the child object and is always declared to be of type object.
 * 
 * -- Global Handlers
 * An asterisk (*) preceeding the handler name signifies a global handler.
 * This handler is at the global scope level and can be called from any
 * movie.
 * 
 * -- Xtra level handlers
 * A plus (+) preceeding the handler name signifies an Xtra level handler. 
 * This kind of handler can be called directly from the Xtra reference,
 * without creating a child object.
 * 
 * The enumerated list that follows must correspond directly with the msgTable 
 * (i.e. they must be in the same order).
 * 
 *******************************************************************************/ 
 
 /* This is the list of handlers for the xtra. The versionInfo string is combined
 /*	with the msgTable string in the register method to create a single string that
 /* used when registering the xtra as a scripting xtra. */

static char versionInfo[] = "xtra ScrnXtra -- version %s.%s.%s\n";
static char msgTable[] = { 
#if TARGET_API_MAC_CARBON
	"-- (for Mac OSX)\n"
#else
	"-- (for Mac Classic)\n"
#endif
	"-- Copyright (c) 1996,97,98 Little Planet Publishing.  All rights reserved.\n"
	"-- For updates see http://www.littleplanet.com/kent/kent.html\n"
	"-- Contact the author at kent@littleplanet.com\n"
	"-- This no-charge Xtra may be freely distributed as long as it is\n"
	"-- accompanied by its documentation and sample movie.\n"
	"--\n"
	"-- Carbonized for Mac OS X by Pierre Luscher in October 2006.\n"
 	"--\n"
 	"new object me\n"  /* standard first handler entry in all message tables */
	"-- Stage Capture Functions --\n"
	"* StageToMember int left, int top, int right, int bottom, any memberNumOrName, * -- any castNumOrName = 1, bool bTrimWhiteSpace = true\n"
	"* StageResizeToMember int left, int top, int right, int bottom, int newWidth, int newHeight, any memberNumOrName, * -- any castNumOrName = 1, bool bTrimWhiteSpace = true\n"
	"* StageToClipboard int left, int top, int right, int bottom\n"
	"* StageResizeToClipboard int left, int top, int right, int bottom, int newWidth, int newHeight\n"
	"* StageToFile int left, int top, int right, int bottom, string filename\n"
	"* StageResizeToFile int left, int top, int right, int bottom, int newWidth, int newHeight, string filename\n"
 	"--\n"
	"-- Screen Capture Functions --\n"
	"* ScreenToMember int left, int top, int right, int bottom, any memberNumOrName, * -- any castNumOrName = 1, bool bTrimWhiteSpace = true\n"
	"* ScreenResizeToMember int left, int top, int right, int bottom, int newWidth, int newHeight, any memberNumOrName, * -- any castNumOrName = 1, bool bTrimWhiteSpace = true\n"
	"* ScreenToClipboard int left, int top, int right, int bottom\n"
	"* ScreenResizeToClipboard int left, int top, int right, int bottom, int newWidth, int newHeight\n"
	"* ScreenToFile int left, int top, int right, int bottom, string filename\n"
	"* ScreenResizeToFile int left, int top, int right, int bottom, int newWidth, int newHeight, string filename\n"
};


/* 	This is the enumerated scripting method list. This list should
 *	directly correspond to the msgTable defined above. It is used
 *	to dispatch method calls via the methodSelector. The 'm_XXXX' method must
 *	be last.
 */

enum 
{
	m_new = 0,						/* standard */
	m_StageToMember,
	m_StageResizeToMember,
	m_StageToClipboard,
	m_StageResizeToClipboard,
	m_StageToFile,
	m_StageResizeToFile,
	m_ScreenToMember,
	m_ScreenResizeToMember,
	m_ScreenToClipboard,
	m_ScreenResizeToClipboard,
	m_ScreenToFile,
	m_ScreenResizeToFile, 

	m_XXXX
};


/* ============================================================================= */
/* Xtra Glue Stuff */
/* ============================================================================= */

#define XTRA_VERSION_NUMBER XTRA_CLASS_VERSION

BEGIN_XTRA
	BEGIN_XTRA_DEFINES_CLASS(ScrnXtra, XTRA_CLASS_VERSION)
		CLASS_DEFINES_INTERFACE(ScrnXtra, IMoaRegister, XTRA_VERSION_NUMBER)
		CLASS_DEFINES_INTERFACE(ScrnXtra, IMoaMmXScript, XTRA_VERSION_NUMBER)
		/*
		 * ---> insert additional interface(s) -->
		 */ 
	END_XTRA_DEFINES_CLASS
END_XTRA


/* ============================================================================= */
/* Create/Destroy for class ScrnXtra */
/* ============================================================================= */


STDMETHODIMP_(MoaError) MoaCreate_ScrnXtra (ScrnXtra FAR * This)
{
moa_try
		
	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaMmValue, (PPMoaVoid) &This->pValueInterface));
	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaMmUtils2, (PPMoaVoid) &This->pMoaUtils));
	

	//-- PL------------
	//MoaChar string[10];
	//CMMReg regWrapper;
	
	// Rcup des Interfaces qui nous intressent

	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaHandle, (PPMoaVoid)&This->pHandle));
	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaAppInfo, (PPMoaVoid)&This->pAppInfo));
	
	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaMmValue, (PPMoaVoid)&This->pMmValue));
	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaDrPlayer, (PPMoaVoid)&This->pDrPlayer));

	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaMmUtils2, (PPMoaVoid)&This->pMmUtils2));
	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaMmList, (PPMoaVoid)&This->pMmList));
	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaDrUtils, (PPMoaVoid)&This->pDrUtils));
	
	//pour MyAlert()
	gMmValue = This->pMmValue;
	gDrPlayer = This->pDrPlayer;

	// Creation des objets de l'xtra
	
	/*** inutile ici
	This->pAppInfo->GetInfo(kMoaAppInfo_RunMode, string, 10);
	if (strcmp(string, "Author")	
		|| regWrapper.IsRegistered("ScrnXtra", "{AD575337L-0003-F443-0000-08000057FC60}")
		)
		This->m_isAllowed = -1; // Ne pas utiliser TRUE/FALSE car c'est Windows Specific
	else
		This->m_isAllowed =  0;
	***/
	//--/PL-------------
	
moa_catch
moa_catch_end
moa_try_end
}

STDMETHODIMP_(void) MoaDestroy_ScrnXtra(ScrnXtra FAR * This)
{
moa_try

	if (This->pValueInterface != NULL) 
		ThrowErr (This->pValueInterface->Release());
		
	if (This->pMoaUtils != NULL) 
		ThrowErr (This->pMoaUtils->Release());
		
	//-- PL------------
	// Release des interfaces utilises
	This->pHandle->Release();
	This->pAppInfo->Release();
	This->pMmValue->Release();
	This->pDrPlayer->Release();
	This->pMmUtils2->Release();

	This->pMmList->Release();
	This->pDrUtils->Release();

	This->pHandle = NULL;
	This->pAppInfo = NULL;
	This->pMmValue = NULL;
	This->pDrPlayer = NULL;
	This->pMmUtils2 = NULL;
	This->pMmList = NULL;
	This->pDrUtils = NULL;
	//--/PL-------------

moa_catch
moa_catch_end
moa_try_end_void
}


/* ============================================================================= */
/* Methods in ScrnXtra_IMoaRegister */
/* ============================================================================= */

/*****************************************************************************
 *  Data needed for Registering
 *	---------------------------
 *	Specific code needed to register different types of Xtras.  The skeleton
 *	source files include minimal implementations appropriate for each Xtra
 *	type.  Current necessary actions:
 *
 *	Scripting Xtra:				Add the Scripting Xtra Message Table
 *	Sprite Asset Xtra:			Nothing
 *	Tool Xtra:					Nothing
 *	Transition Asset Xtra		Nothing
 *
 *  ****optional: Register as Safe for Shockwave!
 *****************************************************************************/ 

STD_INTERFACE_CREATE_DESTROY(ScrnXtra, IMoaRegister)

BEGIN_DEFINE_CLASS_INTERFACE(ScrnXtra, IMoaRegister)
END_DEFINE_CLASS_INTERFACE

/* ----------------------------------------------------------------------------- */
STDMETHODIMP ScrnXtra_IMoaRegister::Register(
	PIMoaCache pCache, 
	PIMoaXtraEntryDict pXtraDict
)
{	
moa_try
	PIMoaRegistryEntryDict	pReg;
	MoaBool					bItsSafe = TRUE;
	char 					versionStr[256];
	PMoaVoid 				pMemStr = NULL;
	
	/***
	//-- PL inutile ici
	PIMoaAppInfo			pAppInfo;
	CMMReg					regWrapper;
	MoaChar					string[10];
	***/
	//pObj->pCallback->QueryInterface(&IID_IMoaAppInfo, (PPMoaVoid)&/*This->*/pAppInfo);
	/***
	pAppInfo->GetInfo(kMoaAppInfo_RunMode, string, 10);
	
	
	if (strcmp(string, "Author") 		  
	|| regWrapper.IsRegistered("ScrnXtra", "{AD575337L-0003-F443-0000-08000057FC60}"))
	{
	***/
	
		// Register the lingo xtra
		ThrowErr(pCache->AddRegistryEntry(pXtraDict, &CLSID_ScrnXtra, &IID_IMoaMmXScript, &pReg));

		// Register the method table
		sprintf(versionStr, versionInfo, VER_MAJORVERSION_STRING, VER_MINORVERSION_STRING, VER_BUGFIXVERSION_STRING);
		pMemStr = pObj->pCalloc->NRAlloc(sizeof(msgTable) + sizeof(versionStr));
		ThrowNull(pMemStr);

		strcpy((char *)pMemStr, versionStr);
		strcat((char *)pMemStr, msgTable);

		ThrowErr(pReg->Put(kMoaDrDictType_MessageTable, pMemStr, 0, kMoaDrDictKey_MessageTable));

		// Mark xtra as safe for shockwave - but only if it IS safe !
		ThrowErr(pReg->Put(kMoaMmDictType_SafeForShockwave, &bItsSafe, sizeof(bItsSafe), kMoaMmDictKey_SafeForShockwave));
	
	/***	
	}
	else
	{	
		MyAlert("Xtra ScrnXtra is not registred.");
	}
	
	// Release des interfaces utilises
	pAppInfo->Release();
	***/
	//--/PL------------

moa_catch
moa_catch_end
	if (pMemStr)
		pObj->pCalloc->NRFree(pMemStr);
moa_try_end
}



/* ============================================================================= */
/*  Methods in ScrnXtra_IMoaMmXScript */
/* ============================================================================= */

BEGIN_DEFINE_CLASS_INTERFACE(ScrnXtra, IMoaMmXScript)
END_DEFINE_CLASS_INTERFACE

//******************************************************************************
ScrnXtra_IMoaMmXScript::ScrnXtra_IMoaMmXScript(MoaError FAR * pError)
//------------------------------------------------------------------------------
{
	*pError = kMoaErr_NoErr;
}	

//******************************************************************************
ScrnXtra_IMoaMmXScript::~ScrnXtra_IMoaMmXScript()
//------------------------------------------------------------------------------
{
}


/* ----------------------------------------------------------------------------- */
STDMETHODIMP ScrnXtra_IMoaMmXScript::Call (PMoaDrCallInfo callPtr)
{
moa_try

	MoaError	err = kMoaErr_NoErr;

	//if (pObj->m_isAllowed) //-- PL inutile ici
	//{
		switch	( callPtr->methodSelector ) 
		{
		
		case m_new:
			{
				// Setup any instance vars for you Xtra here. new() is
				//called via Lingo when creating a new instance.
				// --> insert additional code -->
			}
			break;
		
						
			/* Here is where new methods are added to the switch statement. Each
			   method should be defined in the msgTable defined in and have a 
			   constant defined in the associated enum. 
			*/  

		case m_StageToMember:
			/***
			err = XPly_CaptureMethod(This, callPtr,
							SX_CAPTURE_STAGE, SX_WRITE_MEMBER, SX_NORMAL_IMAGE);
			***/
			pObj->m_captureSource = SX_CAPTURE_STAGE;
			pObj->m_captureDest = SX_WRITE_MEMBER;
			pObj->m_imageSize = SX_NORMAL_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;

		case m_StageResizeToMember:
			pObj->m_captureSource = SX_CAPTURE_STAGE;
			pObj->m_captureDest = SX_WRITE_MEMBER;
			pObj->m_imageSize = SX_RESIZE_IMAGE;
			err = XScrpGlobal_CaptureMethod( callPtr);
			break;
		case m_StageToClipboard:
			pObj->m_captureSource = SX_CAPTURE_STAGE;
			pObj->m_captureDest = SX_WRITE_CLIPBOARD;
			pObj->m_imageSize = SX_NORMAL_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;

		case m_StageResizeToClipboard:
			pObj->m_captureSource = SX_CAPTURE_STAGE;
			pObj->m_captureDest = SX_WRITE_CLIPBOARD;
			pObj->m_imageSize = SX_RESIZE_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;
		
		case m_StageToFile:
			pObj->m_captureSource = SX_CAPTURE_STAGE;
			pObj->m_captureDest = SX_WRITE_FILE;
			pObj->m_imageSize = SX_NORMAL_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;
		
		case m_StageResizeToFile:
			pObj->m_captureSource = SX_CAPTURE_STAGE;
			pObj->m_captureDest = SX_WRITE_FILE;
			pObj->m_imageSize = SX_RESIZE_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;
		
		case m_ScreenToMember:
			pObj->m_captureSource = SX_CAPTURE_SCREEN;
			pObj->m_captureDest = SX_WRITE_MEMBER;
			pObj->m_imageSize = SX_NORMAL_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;
		
		case m_ScreenResizeToMember:
			pObj->m_captureSource = SX_CAPTURE_SCREEN;
			pObj->m_captureDest = SX_WRITE_MEMBER;
			pObj->m_imageSize = SX_RESIZE_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;

		case m_ScreenToClipboard:
			pObj->m_captureSource = SX_CAPTURE_SCREEN;
			pObj->m_captureDest = SX_WRITE_CLIPBOARD;
			pObj->m_imageSize = SX_NORMAL_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;

		case m_ScreenResizeToClipboard:
			pObj->m_captureSource = SX_CAPTURE_SCREEN;
			pObj->m_captureDest = SX_WRITE_CLIPBOARD;
			pObj->m_imageSize = SX_RESIZE_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;
		
		case m_ScreenToFile:
			pObj->m_captureSource = SX_CAPTURE_SCREEN;
			pObj->m_captureDest = SX_WRITE_FILE;
			pObj->m_imageSize = SX_NORMAL_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;
		
		case m_ScreenResizeToFile:
			pObj->m_captureSource = SX_CAPTURE_SCREEN;
			pObj->m_captureDest = SX_WRITE_FILE;
			pObj->m_imageSize = SX_RESIZE_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;

			/*
			 * --> insert additional methodSelector cases -->
			 */

		}
/***
	}
	else
	{
#ifdef _WINDOWS
		MessageBox(NULL, "Registration error", "Xtras' Security", MB_OK|MB_APPLMODAL);
#else
		MyAlert("Xtra ScrnXtra is not registred !");
#endif
	}
***/

moa_catch
moa_catch_end
moa_try_end
}


/*****************************************************************************
 *  Private Methods
 *  -------------------
 *  Implementation of Private Class Methods
 /*
 *  This is the actual code for the defined methods. They are defined as 
 *  functions, and called from the switch statement in Call().
 *
 ****************************************************************************/
/***
static MoaError	XPly_CaptureMethod(CScrnUtilXtra_IMoaDrXLingo FAR *This,
										PMoaDrCallInfo callPtr,
										long captureSource,
										long captureDest,
										long imageSize)
***/

MoaError ScrnXtra_IMoaMmXScript::XScrpGlobal_CaptureMethod(PMoaDrCallInfo callPtr)
{
moa_try

	MoaMmValue	arg1, arg2, arg3, arg4, arg5, arg6, arg7, argMember, argCast;
	int retVal, argCastGiven;
	long rleft, rtop, rright, rbottom, newWidth, newHeight;
	char memberName[256], castName[256];  
	MoaChar	filename[PROPBUF_SIZE];
	PIMoaDrCast pCast;
	MoaMmSymbol formatSymbol, labelSymbol, typeSymbol;
	MoaMmValueType argType;
	MoaDrMediaInfo mediaInfo;
	MoaDrCastIndex castIndex;
	MoaDrMemberIndex memberIndex;   
	PIMoaDrCastMem pCastMem;
	PIMoaMmGC pGC;
	MoaError err;
	MoaMmNativeGCInfo nativeGC;
	PIMoaDrMovie   pMovie;//-- PL
	
#ifdef MACINTOSH
	CGrafPtr stagePort;
	PicHandle picHdl;
#else
	HDC stageDC;
	HBITMAP hBitmap;
	HANDLE hDIB;
	HPALETTE hPal;
#endif
	//-- PL
	MoaMmValue argTrimWhiteSpace;
	MoaLong bTrimWhiteSpace;
	MoaLong lOptions;
	//--/PL

	//UNUSED(This);

//globalThis = This; //-- PL ??? non utilis

	err = pObj->pDrPlayer->GetActiveMovie(&pMovie);//-- PL

	GetArgByIndex(1, &arg1);
	GetArgByIndex(2, &arg2);
	GetArgByIndex(3, &arg3);
	GetArgByIndex(4, &arg4);
	err = pObj->pMmValue->ValueToInteger(&arg1, &rleft);	// convert arguments to longs
	err = pObj->pMmValue->ValueToInteger(&arg2, &rtop);
	err = pObj->pMmValue->ValueToInteger(&arg3, &rright);
	err = pObj->pMmValue->ValueToInteger(&arg4, &rbottom);

	argCastGiven = 0;						// 1 if cast # or name passed, 0 if not

	if(pObj->m_imageSize == SX_NORMAL_IMAGE) {
		newWidth = rright - rleft;			// same size; no change
		newHeight = rbottom - rtop;
	}
	else {
		GetArgByIndex(5, &arg5);			// fetch new width, height arguments
		GetArgByIndex(6, &arg6);
		err = pObj->pMmValue->ValueToInteger(&arg5, &newWidth);
		err = pObj->pMmValue->ValueToInteger(&arg6, &newHeight);
	}

	switch(pObj->m_captureSource) {
		case SX_CAPTURE_STAGE:
			// grab the Graphics Context info	
			//MOVIE(GetStageWindowGC)(MOVARG_ &pGC);
			err = pMovie->GetStageWindowGC(&pGC);//-- PL
			
			//pGC->lpVtbl->GetNativeGCInfo(pGC, &nativeGC);
			pGC->GetNativeGCInfo(&nativeGC);//-- PL
			
		#ifdef MACINTOSH
			if(nativeGC.type == kMoaMmGCType_MacWindow)
				//stagePort = (CGrafPtr) nativeGC.data.mac_window.windowPtr;
				stagePort = GetWindowPort((WindowPtr)nativeGC.data.mac_window.windowPtr); //-- PL
			else if(nativeGC.type == kMoaMmGCType_MacGWorld)
				stagePort = (CGrafPtr) nativeGC.data.mac_gworld.gworldPtr;
			else {
				retVal = -105;
				break;
			}
			retVal = GrabRect(stagePort, rleft, rtop, rright, rbottom,
							newWidth, newHeight, &picHdl);
		#else
			stageDC = (HDC) nativeGC.data.win_window.hDC;
			retVal = GrabRect(stageDC, rleft, rtop, rright, rbottom,
							newWidth, newHeight, &hBitmap, &hDIB, &hPal, stageDC);
		#endif
			//pGC->lpVtbl->Release(pGC);
			pGC->Release();//-- PL
			
			break;

		case SX_CAPTURE_SCREEN:
			// grab the Graphics Context info	
		#ifdef MACINTOSH
			retVal = GrabRect(0, rleft, rtop, rright, rbottom,
							newWidth, newHeight, &picHdl);
		#else
			//MOVIE(GetStageWindowGC)(MOVARG_ &pGC);
			err = pMovie->GetStageWindowGC( &pGC); //-- PL
			
			//pGC->lpVtbl->GetNativeGCInfo(pGC, &nativeGC);
			pGC->GetNativeGCInfo( &nativeGC);
			
			stageDC = (HDC) nativeGC.data.win_window.hDC;
			retVal = GrabRect(0, rleft, rtop, rright, rbottom,
							newWidth, newHeight, &hBitmap, &hDIB, &hPal, stageDC);
			//pGC->lpVtbl->Release(pGC);
			pGC->Release();
		#endif
			break;
	}

	if(retVal != 0) {								// problems during the capture, bail out
		//MmUtils_IntegerToValue(retVal, &callPtr->resultValue);
		pObj->pMmValue->IntegerToValue(retVal, &callPtr->resultValue);//-- PL
		
		return(0);
	}

	switch(pObj->m_captureDest) {
		case SX_WRITE_CLIPBOARD:
		#ifdef MACINTOSH
		
		#if TARGET_API_MAC_CARBON //-- PL
			if(ClearCurrentScrap() != noErr)
		#else
			if(ZeroScrap() != noErr)
		#endif
			{
				retVal = -102;
				break;
			}
			HLock((Handle) picHdl);
			
		#if TARGET_API_MAC_CARBON //-- PL
			ScrapRef scrap;
			GetCurrentScrap(&scrap);
			
			if(PutScrapFlavor(scrap, (ResType) 'PICT', kScrapFlavorMaskNone, kScrapFlavorSizeUnknown, (Ptr) *picHdl) != noErr)
		#else
			if(PutScrap(GetHandleSize((Handle) picHdl), (ResType) 'PICT', (Ptr) *picHdl) != noErr)
		#endif
			{
				retVal = -103;
			}
			HUnlock((Handle) picHdl);
			
		#else
			if(OpenClipboard(GetDesktopWindow())) {
				retVal = 0;
				if(EmptyClipboard()) {
					if(SetClipboardData(CF_DIB, CopyHandle((HANDLE) hDIB)) == NULL)
						retVal = -206;
				}
				else
					retVal = -205;
				if(!CloseClipboard())
					retVal = -207;
			}
			else
				retVal = -204;
		#endif
			break;

		case SX_WRITE_FILE:
			if(pObj->m_imageSize == SX_NORMAL_IMAGE) {
				GetArgByIndex(5, &arg5);
				err = pObj->pMmValue->ValueToString(&arg5, filename, PROPBUF_SIZE);
			}
			else {
				GetArgByIndex(7, &arg7);
				err = pObj->pMmValue->ValueToString(&arg7, filename, PROPBUF_SIZE);
			}

		#ifdef MACINTOSH
			retVal = WritePICT(filename, picHdl);
		#else
			retVal = WriteDIB((PMoaChar) filename, hDIB);
		#endif
			break;

		case SX_WRITE_MEMBER:
			bTrimWhiteSpace = true; //--P PL default value, like previous version
			
			if(pObj->m_imageSize == SX_NORMAL_IMAGE) {
				GetArgByIndex(5, &argMember);
				if(callPtr->nargs > 5) {	// cast name or number is optional
					argCastGiven = 1;
					GetArgByIndex(6, &argCast);
				}
				if (callPtr->nargs > 6) {	// cast name or number is optional
					GetArgByIndex(7, &argTrimWhiteSpace);
					pObj->pValueInterface->ValueToInteger(&argTrimWhiteSpace, &bTrimWhiteSpace);
				}
			}
			else {
				GetArgByIndex(7, &argMember);
				if(callPtr->nargs > 7) {	// cast name or number is optional
					argCastGiven = 1;
					GetArgByIndex(8, &argCast);
				}
				if (callPtr->nargs > 8) {	// cast name or number is optional
					GetArgByIndex(9, &argTrimWhiteSpace);
					pObj->pValueInterface->ValueToInteger(&argTrimWhiteSpace, &bTrimWhiteSpace);
				}
			}

			if(argCastGiven) {
				err = pObj->pMmValue->ValueType(&argCast, &argType);
				switch(argType) {
					case kMoaMmValueType_Integer:
						err = pObj->pMmValue->ValueToInteger(&argCast, &castIndex);
						break;

					case kMoaMmValueType_String:
						err = pObj->pMmValue->ValueToString(&argCast, castName, 256);
						//err = MOVIE(GetCastIndexFromName)(MOVARG_ castName, &castIndex);
						err = pMovie->GetCastIndexFromName(castName, &castIndex); //-- PL
						
						if(err != kMoaErr_NoErr)
							retVal = -21;
						break;

					default:				// unrecognized type passed in
						retVal = -22;
				}
			}
			else							// no cast specified, assume "Internal"
				castIndex = 1;

			if(retVal != 0)
				break;						// jump out if there was an error

			//err = MOVIE(GetNthCast)(MOVARG_ castIndex, &pCast);
			err = pMovie->GetNthCast(castIndex, &pCast); //-- PL
			
			if(err != kMoaErr_NoErr) {				// invalid cast specified
				retVal = -23;
				break;
			}

			err = pObj->pMmValue->ValueType(&argMember, &argType);	// figure out if member ref is an int or string
			switch(argType) {
				case kMoaMmValueType_Integer:
					err = pObj->pMmValue->ValueToInteger(&argMember, &memberIndex);
					break;

				case kMoaMmValueType_String:
					err = pObj->pMmValue->ValueToString(&argMember, memberName, 256);
					//err = pCast->lpVtbl->GetMemberIndexFromName(pCast, memberName, &memberIndex); 
					err = pCast->GetMemberIndexFromName(memberName, &memberIndex); //-- PL
					 
					if(err != kMoaErr_NoErr) {
						//pCast->lpVtbl->Release(pCast);
						pCast->Release(); //-- PL
						
						retVal = -31;
						break;
					}
					break;

				default:							// unrecognized type passed in
					//pCast->lpVtbl->Release(pCast);
					pCast->Release(); //-- PL
					
					retVal = -32;
					break;
			}

			if(retVal != 0)
				break;						// jump out if there was an error

			err = pObj->pMmValue->StringToSymbol("bitmap", &typeSymbol);
			//if(pCast->lpVtbl->CreateCastMem(pCast, memberIndex, typeSymbol) != kMoaErr_NoErr) 
			if(pCast->CreateCastMem(memberIndex, typeSymbol) != kMoaErr_NoErr) //-- PL
			{
				//pCast->lpVtbl->Release(pCast);
				pCast->Release(); //-- PL
				
				retVal = -41;
				break;
			}
			
			lOptions = kMoaDrImgMediaOpts_ImageDepthAndPal;
			//-- PL
			if (bTrimWhiteSpace)
				lOptions |= kMoaDrImgMediaFlags_DontTrimWhiteEdges;			
			//--/PL

			pObj->pMmValue->StringToSymbol("Image", &labelSymbol);
		#ifdef MACINTOSH
			err = pObj->pMmValue->StringToSymbol("macPICT", &formatSymbol);
			err = pObj->pDrUtils->NewMediaInfo(labelSymbol, formatSymbol,
							(PMoaVoid) picHdl, lOptions,
							(PMoaVoid) NULL, &mediaInfo);
		#else
			err = pObj->pMmValue->StringToSymbol("winDIB", &formatSymbol);
			err = pObj->pDrUtils->NewMediaInfo(labelSymbol, formatSymbol,
							(PMoaVoid) hDIB, lOptions,
							(PMoaVoid) NULL, &mediaInfo);
		#endif
		
			if(err != kMoaErr_NoErr) {
				//pCast->lpVtbl->Release(pCast);
				pCast->Release(); //--PL
				
				retVal = -42;
				break;
			}

			//if(pCast->lpVtbl->GetCastMem(pCast, memberIndex, &pCastMem) != kMoaErr_NoErr) 
			if(pCast->GetCastMem(memberIndex, &pCastMem) != kMoaErr_NoErr) //--PL
			{
				//pCast->lpVtbl->Release(pCast);
				pCast->Release(); //--PL
				
				retVal = -43;
				break;
			}

		#ifdef MACINTOSH
			mediaInfo.mediaData = picHdl;
		#else
			mediaInfo.mediaData = hDIB;
		#endif
		
			//if(pCastMem->lpVtbl->SetMedia(pCastMem, &mediaInfo) != kMoaErr_NoErr) 
			if(pCastMem->SetMedia(&mediaInfo) != kMoaErr_NoErr) //-- PL
			{
				//pCastMem->lpVtbl->Release(pCastMem);
				//pCast->lpVtbl->Release(pCast);
				pCastMem->Release(); //-- PL
				pCast->Release(); //-- PL
				
				retVal = -44;
				break;
			}

			//pCastMem->lpVtbl->Release(pCastMem);
			//pCast->lpVtbl->Release(pCast);
			pCastMem->Release(); //-- PL
			pCast->Release(); //-- PL
				
			break;
	}

	#ifdef MACINTOSH
		KillPicture(picHdl);
	#else
		DeleteObject(hBitmap);
	//	DeleteObject(hPal);
		GlobalFree(hDIB);
	//	GlobalFree(hPal);
	#endif

	pObj->pMmValue->IntegerToValue(retVal, &callPtr->resultValue);
	
	//return(0);
	
moa_catch
moa_catch_end	
moa_try_end	
}

/*** ??? n'est apparemment pas utilis
void GetDirRGB(int ndx, int *r, int *g, int *b)
{
	MoaMmRGBTriple RGBColor;
	CScrnUtilXtra_IMoaDrXLingo FAR *This;

	This = globalThis;
	MmUtils_IndexToRGB((MoaMmColorIndex) ndx, &RGBColor);
	*r = RGBColor.red;
	*g = RGBColor.green;
	*b = RGBColor.blue;
}
***/

void MyAlert(char * szMsg)
{
	MoaMmValue		val = {0};
	MoaMmSymbol		sym;
		
	gMmValue->StringToSymbol("alert", &sym);
	gMmValue->StringToValue(szMsg, &val);
	gDrPlayer->CallHandler(sym, 1, &val, 0);
}