// ScrnMac.c for ScrnXtra
// Kent Kersten, 26-Apr-96
// copyright (c) 1996 by Little Planet Publishing, all rights reserved
// heavily reworked April 1997

//#ifdef	__MWERKS__//-- PL v2.2 no, we use XCode now !

#include <MacWindows.h>
#include <string.h>

#include <Types.h>
#include <Memory.h>
#include <Quickdraw.h>
#include <Scrap.h>
#include <Files.h>
#include <Finder.h>
#include <Errors.h>
#include <unistd.h> //-- PL for getcwd()

//#include "beepimp.h" /* MODIFY to match the project's xxxxIMP.H filename trp */
#include "script.h" //-- PL
#include "scrn.h" //-- PL

//-- PL moved into Scrn.h
/*** 
void ctopstr(char *);					// put prototypes here so CW doesn't complain
int WritePICT(char *, PicHandle);
int	GrabRect(CGrafPtr, long, long, long, long, long, long, PicHandle *);
***/

// internal prototype
PixMapHandle createScreenPixMap(Rect * srcRect, Rect * destRect);
OSStatus split_path(const char* path, FSRef *outParentFolder, HFSUniStr255 *outName);
//--/PL

// Convert the given C-format (null terminate) string to Pascal format in situ.

void ctopstr(char *theString)
{
	int i, len;
	char *ptr;

	for(len = 0, ptr = theString; *ptr; ptr++)
		++len;
	for(i = len; i; i--)
		theString[i] = theString[i-1];
	*theString = (unsigned char) len;
}

void basename(const char * filename, char * base)
{
	const char sep = ':';
	strcpy(base, filename);
	char* p = strchr(base, sep);
	
	if (p != NULL) 
		p = p+1; 
	else 
		p = base;
	strcpy(base, p);
}

void dirname (const char *path, char * dir)
{
	const char sep = ':';
	char *p;
	strcpy(dir, path);
	
	if (dir == NULL || *dir == '\0')
		strcpy(dir, ".");
	
	p = dir + strlen(dir) - 1;
	while (*p == sep)
	{
		if (p == dir)
			return;
		
		*p-- = '\0';
	}
	
	while (p > dir && *p != sep)
		p--;
	
	if (p < dir)
		strcpy(dir, ".");
	else if (p == dir)
	{
		dir[0] = sep;
		dir[1] = '\0';
	}
	else
	{
		*p = '\0';
		strcpy(dir, p);
	}
}


// Write out the given picture to a disk file.
// Return 0 for success, a negative error code for failure.
// possible paths: absoute or relative
// "macintosh hd:parentFolder:moviepath:subFolder:toto.pict"
// ":subFolder:toto.pict"
// "@:subFolder:toto.pict"
// "@/subFolder/toto.pict"
// "./subFolder/toto.pict"


int WritePICT(const char * inFilename, PicHandle picHdl, const char * moviePath)
{
#ifdef __MWERKS__
	//codewaroior for Director < 11
	
		short refNum;
		long totalBytes, fivetwelve = 512;
		//char dummyPictHdr[512] = " This picture was written with ScrnXtra v2.0 by Little Planet Publishing ";
		char dummyPictHdr[512] = " This picture was written with ScrnXtra v2.2 by Little Planet Publishing "; //-- PL

		ctopstr(filename);
	#if TARGET_API_MAC_CARBON //-- PL
		FSSpec spec;
		FSMakeFSSpec(0,0,(ConstStr255Param) filename, &spec);
		FSpCreate(&spec, 'ttxt', 'PICT', smSystemScript);
	#else
		Create((ConstStr255Param) filename, 0, 'ttxt', 'PICT');
	#endif	

	#if TARGET_API_MAC_CARBON //-- PL
		if(FSpOpenDF(&spec, 0, &refNum) != noErr)
	#else	
		if(FSOpen((ConstStr255Param) filename, 0, &refNum) != noErr)
	#endif
		{
			FSClose(refNum);
			return(-10);
		}
		HLock((Handle) picHdl);
		totalBytes = GetHandleSize((Handle) picHdl);
		if(FSWrite(refNum, &fivetwelve, dummyPictHdr) != noErr) 
		{
			HUnlock((Handle) picHdl);
			FSClose(refNum);
			return(-11);
		}
		if(FSWrite(refNum, &totalBytes, *picHdl) != noErr) {
			HUnlock((Handle) picHdl);
			FSClose(refNum);
			return(-11);
		}
		HUnlock((Handle) picHdl);//-- PL
		if(FSClose(refNum) != noErr)
			return(-12);
		
		return(0);
	
#else
	//Xcode for Director >= 11
	
	char filename[256];
	char temp[256];
	strcpy(filename, inFilename);
	strcpy(temp, inFilename);
	
	//add a default extension if necessary
	if (! (strstr(filename, ".pict") || strstr(filename, ".pic") || strstr(filename, ".pct")) )
		strcat(filename, ".pict");
	
	//complete fullpath
	if (filename[0] == ':')
	{
		sprintf(temp, "%s%s", moviePath, filename);
	}
	else if (  ((filename[0] == '@') && (filename[1] == ':'))
			|| ((filename[0] == '@') && (filename[1] == '/'))
			|| ((filename[0] == '.') && (filename[1] == '/')))
	{
		sprintf(temp, "%s%s", moviePath, filename+1);
	}
	else if (! strchr(filename, ':'))
	{
		sprintf(temp, "%s:%s", moviePath, filename);
	}
		
	
	//replace all '/' by ':'
	char* str = strchr(temp, '/');
	while (str)
	{
		str[0] = ':';
		str = strchr(str+1, '/');
	}
	
	strcpy(filename, temp);
	
	// <!> filename is here in utf8 with ':'
	//so convert it in POSIX path to can be used by the new FS functions
	
	CFStringRef aHFSString = CFStringCreateWithCString(kCFAllocatorDefault, filename, kCFStringEncodingUTF8);
	CFURLRef url = CFURLCreateWithFileSystemPath(kCFAllocatorDefault, aHFSString, kCFURLHFSPathStyle, false);
	CFStringRef aPOSIXString = CFURLCopyFileSystemPath(url, kCFURLPOSIXPathStyle);
	char filename2[512];
	CFStringGetCString(aPOSIXString, filename2, 512, kCFStringEncodingUTF8);
		
	//get FSSpec of the file
	FSRef			folderRef;
	HFSUniStr255	name;
	OSErr			err;
	bool			fileExists = false;
	
	err = split_path(filename2, &folderRef, &name);
	if ((err != noErr) and (err != fnfErr))
		return (-10);

	FSRef			fileRef;
	err = FSMakeFSRefUnicode(&folderRef, name.length, name.unicode, kTextEncodingMacUnicode, &fileRef);
	if (err == noErr)
		fileExists = true;
	else if (err != fnfErr)
		return (-10);
	
	//delete the file if already exists
	if (fileExists)
		FSDeleteObject(&fileRef);
	
	//create the file
	FSCatalogInfo	catalogInfo;
	FSSpec			fileSpec;
	err = FSCreateFileUnicode(&folderRef, name.length, name.unicode, kFSCatInfoNone, &catalogInfo, &fileRef, &fileSpec);
	if (err != noErr)
		return (-10);
	
	/*
	//set creator & type
	((FileInfo*)&catalogInfo.finderInfo)->fileCreator='ttxt';
	((FileInfo*)&catalogInfo.finderInfo)->fileType='PICT';
	err = FSSetCatalogInfo(&fileRef, kFSCatInfoFinderInfo, &catalogInfo);
	*/
	
	//open data fork
	HFSUniStr255 dataForkName;
	FSGetDataForkName(&dataForkName);
	short refNum;
	err = FSOpenFork(&fileRef, dataForkName.length, dataForkName.unicode, fsRdWrPerm, &refNum);
	if (err != noErr)
		return (-10);
	
	ByteCount fivetwelve = 512;
	//char dummyPictHdr[512] = " This picture was written with ScrnXtra v2.0 by Little Planet Publishing ";
	char dummyPictHdr[512] = " This picture was written with ScrnXtra v2.2 by Little Planet Publishing "; //-- PL
	
	HLock((Handle) picHdl);
	ByteCount totalBytes = GetHandleSize((Handle) picHdl);
	ByteCount writtenBytes;
	
	err = FSWriteFork(refNum, 0, fsFromStart, fivetwelve, (void*) dummyPictHdr, &writtenBytes);
	if (err != noErr)
	{
		HUnlock((Handle) picHdl);
		FSCloseFork(refNum);
		return (-11);
	}
	err = FSWriteFork(refNum, 0, fsAtMark, totalBytes, (void*) *picHdl, &writtenBytes);
	if (err != noErr)
	{
		HUnlock((Handle) picHdl);
		FSCloseFork(refNum);
		return (-11);
	}
	HUnlock((Handle) picHdl);
	err = FSCloseFork(refNum);
	if (err != noErr)
		return(-12);
	
	return (0);

#endif
}
//--/PL

// Return 0 for success, a negative number for failure.

int	GrabRect(CGrafPtr passedGrafPort, long rleft, long rtop, long rright, long rbottom,
			long newWidth, long newHeight, PicHandle *ptrPicHdl)
{
	Rect 	srcRect, destRect;
	long 	destWidth, destHeight;
	
	srcRect.top = (int) rtop;
	srcRect.left = (int) rleft;
	srcRect.bottom = (int) rbottom;
	srcRect.right = (int) rright;

	destRect.top = 0;
	destRect.left = 0;
	if((newHeight > 0) && (newWidth > 0)) 
	{
		destRect.bottom = (int) newHeight;
		destRect.right = (int) newWidth;
	}
	else 
	{
		destRect.bottom = (int) (rbottom - rtop);
		destRect.right = (int) (rright - rleft);
	}
	destWidth = destRect.right;
	destHeight = destRect.bottom;
	
	
#if TARGET_API_MAC_CARBON //-- PL
	// CGrafPort is now opaque
	// GetCWMgrPort, OpenCPort CloseCPort are no more supported
	
	long 			pixelDepth, myRowBytes;
	CGrafPtr 		sourcePort; 
	CGrafPtr        cgOffScrnPtr;
	GrafPtr 		savePort = nil;
	PixMapHandle	pixHandle = nil;
	PicHandle		thePict = nil;
	Ptr 			offScrnBits;
	RgnHandle 		saveClipRgn;
	
	
	if (passedGrafPort)
	{
		//stage copy
		GetPort(&savePort);	
		sourcePort = passedGrafPort;
		
		cgOffScrnPtr = CreateNewPort();
		
		pixelDepth = (*(GetPortPixMap(sourcePort)))->pixelSize;	// only use longs in these calculations!
		// the next line makes sure we get an even byte count for 'rowBytes'
		myRowBytes = (((destWidth * pixelDepth) + 15) / 16) * 2;
		offScrnBits = NewPtr(myRowBytes * destHeight);
		if(offScrnBits == nil)
			return(-104);
		
		/***
		char temp[256];
		//sprintf(temp, "pixelDepth: %i", pixelDepth);
		sprintf(temp, "destWidth: %i, destHeight: %i", destWidth, destHeight);
		MyAlert(temp);
		sprintf(temp, "myRowBytes: %i", myRowBytes);
		MyAlert(temp);
		***/
		
		(*(GetPortPixMap(cgOffScrnPtr)))->baseAddr = offScrnBits;
		(*(GetPortPixMap(cgOffScrnPtr)))->rowBytes = myRowBytes + 0x8000;	// set top bit
		(*(GetPortPixMap(cgOffScrnPtr)))->bounds = destRect;
		
		SetPort((GrafPtr) cgOffScrnPtr);
		ClipRect(&destRect);
		EraseRect(&destRect);

		SetPort((GrafPtr) sourcePort);
		saveClipRgn = NewRgn();
		GetClip(saveClipRgn);						// preserve previous clipping region
		ClipRect(&srcRect);							// make sure the area we are capturing is available

		CopyBits(GetPortBitMapForCopyBits(sourcePort), 
				GetPortBitMapForCopyBits(cgOffScrnPtr), 
				&srcRect, &destRect, srcCopy, nil);
				
		SetPort((GrafPtr) cgOffScrnPtr);
		thePict = OpenPicture(&destRect);
		if (thePict == nil) 
		{
			SetPort((GrafPtr) sourcePort);
			SetClip(saveClipRgn);
			DisposeRgn(saveClipRgn);
			DisposePtr(offScrnBits);
			DisposePort(cgOffScrnPtr);
			SetPort(savePort);
			return(-101);
		}
		
		//HLock((Handle) thePict);
	
		// now copy it to itself to create the picture
		CopyBits(GetPortBitMapForCopyBits(cgOffScrnPtr), 
				GetPortBitMapForCopyBits(cgOffScrnPtr), 
				&destRect, &destRect, srcCopy, nil);
		ClosePicture();
		
		SetPort((GrafPtr) sourcePort);
		SetClip(saveClipRgn);						// restore previous clipping region
		DisposeRgn(saveClipRgn);
		DisposePtr(offScrnBits);
		DisposePort(cgOffScrnPtr);
		SetPort(savePort);
		
		//HUnlock((Handle) thePict);
		*ptrPicHdl = thePict;
	}
	else
	{
		//screen copy
		GetPort(&savePort);	
		
		pixHandle = createScreenPixMap(&srcRect, &destRect);
		LockPixels(pixHandle);
		
		cgOffScrnPtr = CreateNewPort();
		
		pixelDepth = (*pixHandle)->pixelSize;	// only use longs in these calculations!
		// the next line makes sure we get an even byte count for 'rowBytes'
		myRowBytes = (((destWidth * pixelDepth) + 15) / 16) * 2;
		offScrnBits = NewPtr(myRowBytes * destHeight);
		if(offScrnBits  == nil)
			return(-104);
			
		(*(GetPortPixMap(cgOffScrnPtr)))->baseAddr = offScrnBits;
		(*(GetPortPixMap(cgOffScrnPtr)))->rowBytes = myRowBytes + 0x8000;	// set top bit
		(*(GetPortPixMap(cgOffScrnPtr)))->bounds = destRect;
		
		SetPort((GrafPtr) cgOffScrnPtr);
		ClipRect(&destRect);
		EraseRect(&destRect);
		
		thePict = OpenPicture(&destRect);
		if (thePict == nil) 
		{
			DisposePtr(offScrnBits);
			DisposePort(cgOffScrnPtr);
			SetPort(savePort);
			return(-101);
		}
		
		//HLock((Handle) thePict);
	
		CopyBits((BitMap*) *pixHandle, 
				GetPortBitMapForCopyBits(cgOffScrnPtr),
				&destRect, &destRect, srcCopy, 0L);
	 
		ClosePicture();
		DisposePtr(offScrnBits);
		DisposePort(cgOffScrnPtr);
		UnlockPixels(pixHandle);
		SetPort(savePort);
		
		//HUnlock((Handle) thePict);
		*ptrPicHdl = thePict;
	}
	
#else
	// Mac Classic, comme avant
	
	long 		pixelDepth, myRowBytes;
	PicHandle 	thePict;
	Ptr 		offScrnBits;
	GrafPtr 	savePort;
	RgnHandle 	saveClipRgn;
	CGrafPtr 	sourcePort, cgOffScrnPtr;
	CGrafPort 	cgOffScrnPort;
	
	GetPort(&savePort);
	
	if(passedGrafPort)
		sourcePort = passedGrafPort;			// use passed-in graphics port
	else
		GetCWMgrPort(&sourcePort);				// use desktop window port

	cgOffScrnPtr = &cgOffScrnPort;				// create our off-screen grafport
	OpenCPort(cgOffScrnPtr);

	pixelDepth = (*(sourcePort->portPixMap))->pixelSize;	// only use longs in these calculations!
	// the next line makes sure we get an even byte count for 'rowBytes'
	myRowBytes = (((destWidth * pixelDepth) + 15) / 16) * 2;
	if((offScrnBits = NewPtr(myRowBytes * destHeight)) == nil)
		return(-104);
		
	(*(cgOffScrnPtr->portPixMap))->baseAddr = offScrnBits;
	(*(cgOffScrnPtr->portPixMap))->rowBytes = myRowBytes + 0x8000;	// set top bit
	(*(cgOffScrnPtr->portPixMap))->bounds = destRect;
	
	SetPort((GrafPtr) cgOffScrnPtr);
	ClipRect(&destRect);
	EraseRect(&destRect);

	SetPort((GrafPtr) sourcePort);
	saveClipRgn = NewRgn();
	GetClip(saveClipRgn);						// preserve previous clipping region
	ClipRect(&srcRect);							// make sure the area we are capturing is available

	CopyBits((BitMap *) *(sourcePort->portPixMap), (BitMap *) *(cgOffScrnPtr->portPixMap),
				&srcRect, &destRect, srcCopy, nil);	// blap


	SetPort((GrafPtr) cgOffScrnPtr);
	if ((thePict = OpenPicture(&destRect)) == nil) {
		SetPort((GrafPtr) sourcePort);
		SetClip(saveClipRgn);
		DisposeRgn(saveClipRgn);
		DisposePtr(offScrnBits);
		CloseCPort(cgOffScrnPtr);
		SetPort(savePort);
		return(-101);
	}

	HLock((Handle) thePict);
	
	// now copy it to itself to create the picture
	CopyBits((BitMap *) *(cgOffScrnPtr->portPixMap), (BitMap *) *(cgOffScrnPtr->portPixMap),
				&destRect, &destRect, srcCopy, nil);	// blap
	ClosePicture();
	
	SetPort((GrafPtr) sourcePort);
	SetClip(saveClipRgn);						// restore previous clipping region
	DisposeRgn(saveClipRgn);
	DisposePtr(offScrnBits);
	CloseCPort(cgOffScrnPtr);
	SetPort(savePort);

	HUnlock((Handle) thePict);
	*ptrPicHdl = thePict;
	
#endif
	
	return(0);
}

//-- PL like in the example Apple SuperSnapShot :
PixMapHandle createScreenPixMap(Rect * pSrcRect, Rect * pDestRect)
{
	GDHandle	mainDevice;
	CTabHandle	cTable;
	short		depth;	
	Ptr			offBaseAddr;	/* Pointer to the off-screen pixel image */
	short		bytesPerRow;
	PixMapHandle	pixHandle = nil;
	//Rect		bounds;
	
	pixHandle = NewPixMap();
		
	/* Get a handle to the main device. */
	mainDevice = GetMainDevice();

	/* Store its current pixel depth. */
	depth = (**(**mainDevice).gdPMap).pixelSize;

	/* Make an identical copy of its pixmap's colortable. */
	cTable = (**(**mainDevice).gdPMap).pmTable;
	(void) HandToHand( (Handle*)&cTable );
	
	// Get Resolution of screen
	/***
	BitMap bitMap;
	GetQDGlobalsScreenBits(&bitMap);
	SetRect( &bounds, 0, 0, bitMap.bounds.right, bitMap.bounds.bottom);
	***/
	
	// Fill in a few of the PixMap's fields...
	// NewPixMap() is good for default initialization, simply modify
	// the new PixMap
	(*pixHandle)->pmTable = cTable;
	//(*pixHandle)->bounds = *pSrcRect;
	(*pixHandle)->bounds = *pDestRect;//-- PL v2.1.1
	
	(*pixHandle)->pixelSize = depth;
	bytesPerRow = ((pSrcRect->right - pSrcRect->left) * depth) / 8;
	offBaseAddr = NewPtr((unsigned long) bytesPerRow * (pSrcRect->bottom - pSrcRect->top));
	(*pixHandle)->baseAddr = offBaseAddr;  			// Point to image
	(*pixHandle)->rowBytes = bytesPerRow | 0x8000;	// MSB set for PixMap
	
	LockPixels(pixHandle);
	
	CopyBits( (BitMap *)*(**mainDevice).gdPMap, (BitMap *) *pixHandle,
				pSrcRect, pDestRect, srcCopy, nil );
	
	UnlockPixels(pixHandle);
	
	return pixHandle;
}

//Use POSIX path (utf8 with /)
OSStatus split_path(const char* path, FSRef *outParentFolder, HFSUniStr255 *outName)
{
	char            tmpPath[ PATH_MAX ];
	char            tmpName[ PATH_MAX ];
	char			*tmpNamePtr;
	OSErr           err;
	
	/* Get local copy of incoming path                          */
	strcpy( tmpPath, (char*)path );
	
	/* Get the name of the object from the given path     */
	/* Find the last : and change it to a '\0' so         */
	/* tmpPath is a path to the parent directory of the   */
	/* object and tmpNamePtr is the name                        */
	tmpNamePtr = strrchr( tmpPath, '/' );
	if ( tmpNamePtr == NULL )
	{
		getcwd(tmpPath, sizeof(tmpPath));
	}
	else
	{
		if( *(tmpNamePtr + 1) == '\0' )
		{                       /* in case the last character in the path is a /      */
			*tmpNamePtr = '\0';
			tmpNamePtr = strrchr( tmpPath, '/' );
		}
		*tmpNamePtr = '\0';
		tmpNamePtr++;
		
		strcpy(tmpName, tmpNamePtr);
		tmpNamePtr = tmpName;
		
		if ( tmpPath[0] == '\0' )
			getcwd(tmpPath, sizeof(tmpPath));
	}
	
	/* Get the FSRef to the parent directory              */
	err = FSPathMakeRef( (unsigned char*)tmpPath, outParentFolder, NULL );
	if( err == noErr )
	{                       /* Convert the name to an HFSUniStr255 */
		CFStringRef       tmpStringRef = CFStringCreateWithCString( kCFAllocatorDefault, tmpNamePtr ? tmpNamePtr : path, kCFStringEncodingUTF8 );
		if( tmpStringRef != NULL )
		{
			outName->length = CFStringGetLength(tmpStringRef);
			if ( outName->length > sizeof(outName->unicode)/sizeof(UniChar) )
				err = bdNamErr;
			
			if ( err == noErr ) 
				CFStringGetCharacters(tmpStringRef, CFRangeMake(0, outName->length), outName->unicode);
			
			CFRelease( tmpStringRef );
		}
		else err = coreFoundationUnknownErr;
	}
	
	return err;
}


//--/PL

//#endif		// __MWERKS__//-- PL
