/*
ADOBE SYSTEMS INCORPORATED
Copyright 1994 - 2008 Adobe Macromedia Software LLC
All Rights Reserved

NOTICE:  Adobe permits you to use, modify, and distribute this file in accordance with the 
terms of the Adobe license agreement accompanying it.  If you have received this file from a 
source other than Adobe, then your use, modification, or distribution of it requires the prior 
written permission of Adobe.
*/


#define INITGUID 1

#include "../source/dversion.h" //for the version number
#include "script.h"

#include <wchar.h> //-- PL
#include <string.h>
#include <stdlib.h>
#include "xclassver.h"
#include "moatry.h"

#include "driservc.h"
#include "drivalue.h"

#include "mmivalue.h"
#include "mmillist.h"
#include "mmiplist.h"
#include "mmidate.h"
#include "mmiclr.h"

//-- PL--------------
#ifdef MACINTOSH
	#include <windows.h>
	#include <string.h>
	//#include <stdlib.h> // Malloc/Free
	#include <scrap.h>
#endif

#include "scrn.h"

#define PROPBUF_SIZE	256
#define PROPNAME_SIZE	32

#define	SX_CAPTURE_STAGE	1			// sources for screen captures
#define	SX_CAPTURE_SCREEN	2

#define	SX_NORMAL_IMAGE		1			// tells whether to stretch the image or not
#define	SX_RESIZE_IMAGE		2

#define	SX_WRITE_MEMBER		1			// destinations for screen captures
#define	SX_WRITE_CLIPBOARD	2
#define	SX_WRITE_FILE		3

/* Global to hold temporary argument values */
//static	MoaMmValue	gTempArg;

// globales pour MyAlert()
PIMoaMmValue gMmValue;
PIMoaDrPlayer gDrPlayer;

//--/PL---------------

/*******************************************************************************
 * SCRIPTING XTRA MESSAGE TABLE DESCRIPTION.
 *
 * The general format is:
 * xtra <nameOfXtra>
 * new object me [ args ... ]
 * <otherHandlerDefintions>
 * --
 * The first line must give the name of the Scripting xtra.
 * The remaining lines give the names of the handlers that this xtra implements
 * along with the required types of the arguments for each handler.
 * 
 * -- Pre-defined handler new 
 * The new handler will be called when a child object is created, 
 * the first argument is always the child object and you defined any remaining arguments.
 * The new handler is the place to initialize any memory required by the child object.
 * 
 * -- Simple Handler Definitions
 * Each handler definition line is format like this:
 * <handlerName> <argType1> <argName1>, <argType2> <argName2> ...
 * The first word is the handler name. Following this are types description for
 * the argument to the handler, each separated by a comma.
 * The argument name <argName>, may be omited.
 * Permited argument types are: 
 * 	integer 
 * 	float
 * 	string
 * 	symbol
 * 	object
 * 	any
 * 	*
 * For integer, float, string, symbol, and object, the type of the argument must 
 * match. The type any means allow any type. The asterisk (*) means any number and 
 * any type of arguments.
 * 
 * The first argument is the child object and is always declared to be of type object.
 * 
 * -- Global Handlers
 * An asterisk (*) preceeding the handler name signifies a global handler.
 * This handler is at the global scope level and can be called from any
 * movie.
 * 
 * -- Xtra level handlers
 * A plus (+) preceeding the handler name signifies an Xtra level handler. 
 * This kind of handler can be called directly from the Xtra reference,
 * without creating a child object.
 * 
 * The enumerated list that follows must correspond directly with the msgTable 
 * (i.e. they must be in the same order).
 * 
 *******************************************************************************/ 
 
 /* This is the list of handlers for the xtra. The versionInfo string is combined
 /*	with the msgTable string in the register method to create a single string that
 /* used when registering the xtra as a scripting xtra. */

static char versionInfo[] = "xtra ScrnXtra -- version %s.%s.%s\n";
static char msgTable[] = { 
	/***
	"new object me\n"  // standard first handler entry in all message tables
 	"-- Template handlers --\n" 
	"* globalHandler -- prints global handler message\n"
	"+ parentHandler  object xtraRef -- prints parent handler message\n" 
	"childHandler object me, integer number -- prints child handler message, returns a number\n"
	***/
		/*
		 * ---> insert additional handler(s) MUST MATCH WITH ENUMS BELOW -->
		 */ 
	"-- Copyright (c) 1996,97,98 Little Planet Publishing.  All rights reserved.\n"
	"-- For updates see http://www.littleplanet.com/kent/kent.html\n"
	"-- Contact the author at kent@littleplanet.com\n"
	"-- This no-charge Xtra may be freely distributed as long as it is\n"
	"-- accompanied by its documentation and sample movie.\n"
	"--\n"
	"-- function ScreenToImage added by Valentin Schmidt\n"
	"--\n"
	"-- Adapted for Director 11 by Pierre Luscher.\n"
 	"--\n"
 	//"new object me\n"  /* standard first handler entry in all message tables */
	"-- Stage Capture Functions --\n"
	"* StageToMember int left, int top, int right, int bottom, any memberNumOrName, * -- any castNumOrName = 1, bool trimWhiteSpace = true\n"
	"* StageResizeToMember int left, int top, int right, int bottom, int newWidth, int newHeight, any memberNumOrName, * -- any castNumOrName = 1, bool trimWhiteSpace = true\n"
	"* StageToClipboard int left, int top, int right, int bottom\n"
	"* StageResizeToClipboard int left, int top, int right, int bottom, int newWidth, int newHeight\n"
	"* StageToFile int left, int top, int right, int bottom, string filename\n"
	"* StageResizeToFile int left, int top, int right, int bottom, int newWidth, int newHeight, string filename\n"
 	"--\n"
	"-- Screen Capture Functions --\n"
	"* ScreenToMember int left, int top, int right, int bottom, any memberNumOrName, * -- any castNumOrName = 1, bool trimWhiteSpace = true\n"
	"* ScreenResizeToMember int left, int top, int right, int bottom, int newWidth, int newHeight, any memberNumOrName, * -- any castNumOrName = 1, bool trimWhiteSpace = true\n"
	"* ScreenToClipboard int left, int top, int right, int bottom\n"
	"* ScreenResizeToClipboard int left, int top, int right, int bottom, int newWidth, int newHeight\n"
	"* ScreenToFile int left, int top, int right, int bottom, string filename\n"
	"* ScreenResizeToFile int left, int top, int right, int bottom, int newWidth, int newHeight, string filename\n"
};



/* 	This is the enumerated scripting method list. This list should
 *	directly correspond to the msgTable defined above. It is used
 *	to dispatch method calls via the methodSelector. The 'm_XXXX' method must
 *	be last.
 */

enum 
{
	//m_new = 0, /* standard first entry */
	/***
	m_globalHandler,						
	m_parentHandler,
	m_childHandler,
	***/
		/*
		 * ---> insert additional names(s) MUST MATCH MESSAGE TABLE ABOVE -->
		 */ 
	m_StageToMember = 0,
	m_StageResizeToMember,
	m_StageToClipboard,
	m_StageResizeToClipboard,
	m_StageToFile,
	m_StageResizeToFile,
	m_ScreenToMember,
	m_ScreenResizeToMember,
	m_ScreenToClipboard,
	m_ScreenResizeToClipboard,
	m_ScreenToFile,
	m_ScreenResizeToFile,
	
	m_XXXX
};


/* ============================================================================= */
/* Xtra Glue Stuff */
/* ============================================================================= */

#define XTRA_VERSION_NUMBER XTRA_CLASS_VERSION

BEGIN_XTRA
	BEGIN_XTRA_DEFINES_CLASS(TStdXtra, XTRA_CLASS_VERSION)
		CLASS_DEFINES_INTERFACE(TStdXtra, IMoaRegister, XTRA_VERSION_NUMBER)
		CLASS_DEFINES_INTERFACE(TStdXtra, IMoaMmXScript, XTRA_VERSION_NUMBER)
		/*
		 * ---> insert additional interface(s) -->
		 */ 
	END_XTRA_DEFINES_CLASS
END_XTRA


/* ============================================================================= */
/* Create/Destroy for class TStdXtra */
/* ============================================================================= */


STDMETHODIMP_(MoaError) MoaCreate_TStdXtra (TStdXtra * This)
{
moa_try
		
	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaMmValue, (PPMoaVoid) &This->pMmValue));
	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaMmUtils2, (PPMoaVoid) &This->pMoaUtils));
	
	//--PL
	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaHandle, (PPMoaVoid)&This->pHandle));
	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaAppInfo, (PPMoaVoid)&This->pAppInfo));
	
	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaMmValue, (PPMoaVoid)&This->pMmValue));
	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaDrPlayer, (PPMoaVoid)&This->pDrPlayer));

	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaMmUtils2, (PPMoaVoid)&This->pMmUtils2));
	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaMmList, (PPMoaVoid)&This->pMmList));
	ThrowErr (This->pCallback->QueryInterface(&IID_IMoaDrUtils, (PPMoaVoid)&This->pDrUtils));
	
	ThrowErr (This->pCallback->MoaCreateInstance(&CLSID_CMoaFile2, &IID_IMoaFile2, (PPMoaVoid)&This->pFile2));
	
	//pour MyAlert()
	gMmValue = This->pMmValue;
	gDrPlayer = This->pDrPlayer;
	//--/PL
	
moa_catch
moa_catch_end
moa_try_end
}

STDMETHODIMP_(void) MoaDestroy_TStdXtra(TStdXtra * This)
{
moa_try

	if (This->pMmValue != NULL) 
		ThrowErr (This->pMmValue->Release());
		
	if (This->pMoaUtils != NULL) 
		ThrowErr (This->pMoaUtils->Release());
		
	//-- PL------------
	
	// Release the used interfaces
	This->pHandle->Release();
	This->pAppInfo->Release();
	This->pMmValue->Release();
	This->pDrPlayer->Release();
	This->pMmUtils2->Release();
	This->pMmList->Release();
	This->pDrUtils->Release();
	This->pFile2->Release();

	This->pHandle = NULL;
	This->pAppInfo = NULL;
	This->pMmValue = NULL;
	This->pDrPlayer = NULL;
	This->pMmUtils2 = NULL;
	This->pMmList = NULL;
	This->pDrUtils = NULL;
	This->pFile2 = NULL;
	//--/PL-------------

moa_catch
moa_catch_end
moa_try_end_void
}


/* ============================================================================= */
/* Methods in TStdXtra_IMoaRegister */
/* ============================================================================= */

/*****************************************************************************
 *  Data needed for Registering
 *	---------------------------
 *	Specific code needed to register different types of Xtras.  The skeleton
 *	source files include minimal implementations appropriate for each Xtra
 *	type.  Current necessary actions:
 *
 *	Scripting Xtra:				Add the Scripting Xtra Message Table
 *	Sprite Asset Xtra:			Nothing
 *	Tool Xtra:					Nothing
 *	Transition Asset Xtra		Nothing
 *
 *  ****optional: Register as Safe for Shockwave!
 *****************************************************************************/ 

STD_INTERFACE_CREATE_DESTROY(TStdXtra, IMoaRegister)

BEGIN_DEFINE_CLASS_INTERFACE(TStdXtra, IMoaRegister)
END_DEFINE_CLASS_INTERFACE

/* ----------------------------------------------------------------------------- */
STDMETHODIMP TStdXtra_IMoaRegister::Register(
	PIMoaCache pCache, 
	PIMoaXtraEntryDict pXtraDict
)
{	
moa_try
	PIMoaRegistryEntryDict		pReg;
	MoaBool						bItsSafe = TRUE;
	char versionStr[256];
	PMoaVoid pMemStr = NULL;

	/* Register the lingo xtra */
	ThrowErr(pCache->AddRegistryEntry(pXtraDict, &CLSID_TStdXtra, &IID_IMoaMmXScript, &pReg));

	/* Register the method table */
	sprintf(versionStr, versionInfo, VER_MAJORVERSION_STRING, VER_MINORVERSION_STRING, VER_BUGFIXVERSION_STRING);
	pMemStr = pObj->pCalloc->NRAlloc(sizeof(msgTable) + sizeof(versionStr));
	ThrowNull(pMemStr);

	strcpy((char *)pMemStr, versionStr);
	strcat((char *)pMemStr, msgTable);

	ThrowErr(pReg->Put(kMoaDrDictType_MessageTable, pMemStr, 0, kMoaDrDictKey_MessageTable));

	/* Mark xtra as safe for shockwave - but only if it IS safe ! */
	ThrowErr(pReg->Put(kMoaMmDictType_SafeForShockwave, &bItsSafe, sizeof(bItsSafe), kMoaMmDictKey_SafeForShockwave));
	

moa_catch
moa_catch_end
	if (pMemStr)
		pObj->pCalloc->NRFree(pMemStr);
moa_try_end
}



/* ============================================================================= */
/*  Methods in TStdXtra_IMoaMmXScript */
/* ============================================================================= */

BEGIN_DEFINE_CLASS_INTERFACE(TStdXtra, IMoaMmXScript)
END_DEFINE_CLASS_INTERFACE

//******************************************************************************
TStdXtra_IMoaMmXScript::TStdXtra_IMoaMmXScript(MoaError * pError)
//------------------------------------------------------------------------------
{
	*pError = kMoaErr_NoErr;
}	

//******************************************************************************
TStdXtra_IMoaMmXScript::~TStdXtra_IMoaMmXScript()
//------------------------------------------------------------------------------
{
}


/* ----------------------------------------------------------------------------- */
STDMETHODIMP TStdXtra_IMoaMmXScript::Call (PMoaDrCallInfo callPtr)
{
moa_try

	MoaError err = kMoaErr_NoErr;
	
	switch	( callPtr->methodSelector ) 
	{
		//case m_new: // not used heree
			//{
			/* Setup any instance vars for you Xtra here. new() is
			called via Lingo when creating a new instance. */
			/*
			 * --> insert additional code -->
		 	 */			
			//}
			//break;
				
		/* Here is where new methods are added to the switch statement. Each
		   method should be defined in the msgTable defined in and have a 
		   constant defined in the associated enum. 
		*/  
		/***
		case m_globalHandler:
			ThrowErr(XScrpGlobalHandler(callPtr));
			break;

		case m_parentHandler:
			ThrowErr(XScrpParentHandler(callPtr));
			break;

		case m_childHandler:
			ThrowErr(XScrpChildHandler(callPtr));
			break;
		***/
		/*
		 * --> insert additional methodSelector cases -->
		 */
		 
		case m_StageToMember:
			//MyAlert("m_StageToMember");
			pObj->m_captureSource = SX_CAPTURE_STAGE;
			pObj->m_captureDest = SX_WRITE_MEMBER;
			pObj->m_imageSize = SX_NORMAL_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;
			
		case m_StageResizeToMember:
			//MyAlert("m_StageResizeToMember");
			pObj->m_captureSource = SX_CAPTURE_STAGE;
			pObj->m_captureDest = SX_WRITE_MEMBER;
			pObj->m_imageSize = SX_RESIZE_IMAGE;
			err = XScrpGlobal_CaptureMethod( callPtr);
			break;
			
		case m_StageToClipboard:
			//MyAlert("m_StageToClipboard");
			pObj->m_captureSource = SX_CAPTURE_STAGE;
			pObj->m_captureDest = SX_WRITE_CLIPBOARD;
			pObj->m_imageSize = SX_NORMAL_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;

		case m_StageResizeToClipboard:
			//MyAlert("m_StageResizeToClipboard");
			pObj->m_captureSource = SX_CAPTURE_STAGE;
			pObj->m_captureDest = SX_WRITE_CLIPBOARD;
			pObj->m_imageSize = SX_RESIZE_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;
		
		case m_StageToFile:
			//MyAlert("m_StageToFile");
			pObj->m_captureSource = SX_CAPTURE_STAGE;
			pObj->m_captureDest = SX_WRITE_FILE;
			pObj->m_imageSize = SX_NORMAL_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;
		
		case m_StageResizeToFile:
			//MyAlert("m_StageResizeToFile");
			pObj->m_captureSource = SX_CAPTURE_STAGE;
			pObj->m_captureDest = SX_WRITE_FILE;
			pObj->m_imageSize = SX_RESIZE_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;
		
		case m_ScreenToMember:
			//MyAlert("m_ScreenToMember");
			pObj->m_captureSource = SX_CAPTURE_SCREEN;
			pObj->m_captureDest = SX_WRITE_MEMBER;
			pObj->m_imageSize = SX_NORMAL_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;
		
		case m_ScreenResizeToMember:
			//MyAlert("m_ScreenResizeToMember");
			pObj->m_captureSource = SX_CAPTURE_SCREEN;
			pObj->m_captureDest = SX_WRITE_MEMBER;
			pObj->m_imageSize = SX_RESIZE_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;

		case m_ScreenToClipboard:
			//MyAlert("m_ScreenToClipboard");
			pObj->m_captureSource = SX_CAPTURE_SCREEN;
			pObj->m_captureDest = SX_WRITE_CLIPBOARD;
			pObj->m_imageSize = SX_NORMAL_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;

		case m_ScreenResizeToClipboard:
			//MyAlert("m_ScreenResizeToClipboard");
			pObj->m_captureSource = SX_CAPTURE_SCREEN;
			pObj->m_captureDest = SX_WRITE_CLIPBOARD;
			pObj->m_imageSize = SX_RESIZE_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;
		
		case m_ScreenToFile:
			//MyAlert("m_ScreenToFile");
			pObj->m_captureSource = SX_CAPTURE_SCREEN;
			pObj->m_captureDest = SX_WRITE_FILE;
			pObj->m_imageSize = SX_NORMAL_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;
		
		case m_ScreenResizeToFile:
			//MyAlert("m_ScreenResizeToFile");
			pObj->m_captureSource = SX_CAPTURE_SCREEN;
			pObj->m_captureDest = SX_WRITE_FILE;
			pObj->m_imageSize = SX_RESIZE_IMAGE;
			err = XScrpGlobal_CaptureMethod(callPtr);
			break;
			
	}
moa_catch
moa_catch_end
moa_try_end
}


/*****************************************************************************
 *  Private Methods
 *  -------------------
 *  Implementation of Private Class Methods
 /*
 *  This is the actual code for the defined methods. They are defined as 
 *  functions, and called from the switch statement in Call().
 *
 ****************************************************************************/
/***
// ----------------------------------------------------- XScrpGlobalHandler
MoaError TStdXtra_IMoaMmXScript::XScrpGlobalHandler(PMoaDrCallInfo pCall)
{
moa_try
	
	ThrowErr(pObj->pMoaUtils->PrintMessage("GlobalHandler called\n"));

	//  --> insert additional code -->
	
moa_catch
moa_catch_end	
moa_try_end	
}

// ----------------------------------------------------- XScrpParentHandler
MoaError TStdXtra_IMoaMmXScript::XScrpParentHandler(PMoaDrCallInfo pCall)
{	
moa_try

	ThrowErr(pObj->pMoaUtils->PrintMessage("ParentHandler called\n"));
	
	//	--> insert additional code -->
	 

moa_catch
moa_catch_end	
moa_try_end	
}

// ------------------------------------------------------ XScrpChildHandler
MoaError TStdXtra_IMoaMmXScript::XScrpChildHandler(PMoaDrCallInfo pCall)
{
	MoaMmValue argValue;
	MoaLong num;
	
moa_try

	// This shows how to access an argument
	//  the first argument in the list is the "me" value, so the user arguments
	//  start at the second position in the list
	pciGetArgByIndex( pCall, 2, &argValue );
	
	ThrowErr (pObj->pValueInterface->ValueToInteger( &argValue, &num));
	ThrowErr (pObj->pMoaUtils->PrintMessage1("ChildHandler called with (%d)\n", num));

	
	 //  --> insert additional code -->
	 

	// if we want the lingo handler to return something do it like this
	
	ThrowErr(pObj->pValueInterface->IntegerToValue(17, &pCall->resultValue));

moa_catch
moa_catch_end	
moa_try_end	
}
***/
//sur mac intel : scrap manager -> pasteboard manager
//PL
MoaError TStdXtra_IMoaMmXScript::XScrpGlobal_CaptureMethod(PMoaDrCallInfo callPtr)
{
moa_try

	MoaMmValue	arg1, arg2, arg3, arg4, arg5, arg6, arg7, argMember, argCast;
	int retVal, argCastGiven;
	long rleft, rtop, rright, rbottom, newWidth, newHeight;
	char memberName[256], castName[256];	
	MoaChar	filename[PROPBUF_SIZE];	
	PIMoaDrCast pCast;
	MoaMmSymbol formatSymbol, labelSymbol, typeSymbol;
	MoaMmValueType argType;
	MoaDrMediaInfo mediaInfo;
	MoaDrCastIndex castIndex;
	MoaDrMemberIndex memberIndex;   
	PIMoaDrCastMem pCastMem;
	PIMoaMmGC pGC;
	MoaError err;
	MoaMmNativeGCInfo nativeGC;
	PIMoaDrMovie   pMovie;//-- PL
	PIMoaPathName	pPathName = NULL;
	char moviePath[MOA_MAX_PATHNAME];
	
	#ifdef MACINTOSH
		CGrafPtr stagePort;
		PicHandle picHdl;
	#else
		HDC stageDC;
		HBITMAP hBitmap;
		HANDLE hDIB;
		HPALETTE hPal;
	#endif
	//-- PL
	MoaMmValue argTrimWhiteSpace;
	MoaLong bTrimWhiteSpace;
	MoaLong lOptions;
	//--/PL

	//UNUSED(This);

//globalThis = This; //-- PL ??? not used

	err = pObj->pDrPlayer->GetActiveMovie(&pMovie);//-- PL

	GetArgByIndex(1, &arg1);
	GetArgByIndex(2, &arg2);
	GetArgByIndex(3, &arg3);
	GetArgByIndex(4, &arg4);
	err = pObj->pMmValue->ValueToInteger(&arg1, &rleft);	// convert arguments to longs
	err = pObj->pMmValue->ValueToInteger(&arg2, &rtop);
	err = pObj->pMmValue->ValueToInteger(&arg3, &rright);
	err = pObj->pMmValue->ValueToInteger(&arg4, &rbottom);

	argCastGiven = 0;						// 1 if cast # or name passed, 0 if not

	if(pObj->m_imageSize == SX_NORMAL_IMAGE) {
		newWidth = rright - rleft;			// same size; no change
		newHeight = rbottom - rtop;
	}
	else {
		GetArgByIndex(5, &arg5);			// fetch new width, height arguments
		GetArgByIndex(6, &arg6);
		err = pObj->pMmValue->ValueToInteger(&arg5, &newWidth);
		err = pObj->pMmValue->ValueToInteger(&arg6, &newHeight);
	}

	switch(pObj->m_captureSource) {
		case SX_CAPTURE_STAGE:
			// grab the Graphics Context info	
			//MOVIE(GetStageWindowGC)(MOVARG_ &pGC);
			err = pMovie->GetStageWindowGC(&pGC);//-- PL
			
			//pGC->lpVtbl->GetNativeGCInfo(pGC, &nativeGC);
			pGC->GetNativeGCInfo(&nativeGC);//-- PL
			
			#ifdef MACINTOSH
				if(nativeGC.type == kMoaMmGCType_MacWindow)
					//stagePort = (CGrafPtr) nativeGC.data.mac_window.windowPtr;
					stagePort = GetWindowPort((WindowPtr)nativeGC.data.mac_window.windowPtr); //-- PL
				else if(nativeGC.type == kMoaMmGCType_MacGWorld)
					stagePort = (CGrafPtr) nativeGC.data.mac_gworld.gworldPtr;
				else {
					retVal = -105;
					break;
				}
				retVal = GrabRect(stagePort, rleft, rtop, rright, rbottom,
								newWidth, newHeight, &picHdl);
			#else
				stageDC = (HDC) nativeGC.data.win_window.hDC;
				retVal = GrabRect(stageDC, rleft, rtop, rright, rbottom,
								newWidth, newHeight, &hBitmap, &hDIB, &hPal, stageDC);
			#endif
			//pGC->lpVtbl->Release(pGC);
			pGC->Release();//-- PL
			
			break;

		case SX_CAPTURE_SCREEN:
			// grab the Graphics Context info	
			#ifdef MACINTOSH
				retVal = GrabRect(0, rleft, rtop, rright, rbottom,
								newWidth, newHeight, &picHdl);
			#else
				//MOVIE(GetStageWindowGC)(MOVARG_ &pGC);
				err = pMovie->GetStageWindowGC( &pGC); //-- PL
				
				//pGC->lpVtbl->GetNativeGCInfo(pGC, &nativeGC);
				pGC->GetNativeGCInfo( &nativeGC);
				
				stageDC = (HDC) nativeGC.data.win_window.hDC;
				retVal = GrabRect(0, rleft, rtop, rright, rbottom,
								newWidth, newHeight, &hBitmap, &hDIB, &hPal, stageDC);
				//pGC->lpVtbl->Release(pGC);
				pGC->Release();
			#endif
			break;
	}

	if(retVal != 0) {								// problems during the capture, bail out
		//MmUtils_IntegerToValue(retVal, &callPtr->resultValue);
		pObj->pMmValue->IntegerToValue(retVal, &callPtr->resultValue);//-- PL
		
		return(0);
	}

	switch(pObj->m_captureDest) {
		case SX_WRITE_CLIPBOARD:
			#ifdef MACINTOSH
				#if TARGET_API_MAC_CARBON //-- PL
					if(ClearCurrentScrap() != noErr)
					//@ PasteboardCreate(kPasteboardClipboard, &pasteboard);
					//@ if (PasteboardClear(pasteboard) = !noErr)
				#else
					if(ZeroScrap() != noErr)
				#endif
					{
						retVal = -102;
						break;
					}
					HLock((Handle) picHdl);
					
				#if TARGET_API_MAC_CARBON //-- PL
					ScrapRef scrap;
					GetCurrentScrap(&scrap);
					
					if (PutScrapFlavor(scrap, (ResType) 'PICT', kScrapFlavorMaskNone, kScrapFlavorSizeUnknown, (Ptr) *picHdl) != noErr)
					//@ if (PasteboardPutItemFlavor(pasteboard, itemid, (ResType) 'PICT', (Ptr) *picHdl, 
				#else
					if (PutScrap(GetHandleSize((Handle) picHdl), (ResType) 'PICT', (Ptr) *picHdl) != noErr)
				#endif
					{
						retVal = -103;
					}
					HUnlock((Handle) picHdl);
					
			#else
					if(OpenClipboard(GetDesktopWindow())) {
						retVal = 0;
						if(EmptyClipboard()) {
							if(SetClipboardData(CF_DIB, CopyHandle((HANDLE) hDIB)) == NULL)
								retVal = -206;
						}
						else
							retVal = -205;
						if(!CloseClipboard())
							retVal = -207;
					}
					else
						retVal = -204;
			#endif
			break;

		case SX_WRITE_FILE:
			if(pObj->m_imageSize == SX_NORMAL_IMAGE) {
				GetArgByIndex(5, &arg5);
				err = pObj->pMmValue->ValueToString(&arg5, filename, PROPBUF_SIZE);
			}
			else {
				GetArgByIndex(7, &arg7);
				err = pObj->pMmValue->ValueToString(&arg7, filename, PROPBUF_SIZE);
			}
			
			#ifdef MACINTOSH
				pObj->pCallback->MoaCreateInstance(&CLSID_CMoaPath, &IID_IMoaPathName, (PPMoaVoid)&pPathName);
				pObj->pFile2->GetPathName(&pPathName);
				pPathName->GetPath(moviePath, MOA_MAX_PATHNAME);
			
				retVal = WritePICT(filename, picHdl, moviePath);
			
				if (pPathName)
				{
					pPathName->Release();
					pPathName = NULL;
				}
			#else
				retVal = WriteDIB((PMoaChar) filename, hDIB);
			#endif
			break;

		case SX_WRITE_MEMBER:
			bTrimWhiteSpace = true; //PL default value, like previous version behavior
			
			if(pObj->m_imageSize == SX_NORMAL_IMAGE) {
				GetArgByIndex(5, &argMember);
				if(callPtr->nargs > 5) {	// cast name or number is optional
					argCastGiven = 1;
					GetArgByIndex(6, &argCast);
				}
				//-- PL
				if (callPtr->nargs > 6) {
					GetArgByIndex(6, &argTrimWhiteSpace);
					pObj->pValueInterface->ValueToInteger(&argTrimWhiteSpace, &bTrimWhiteSpace);
				}
				//--/PL
			}
			else {
				GetArgByIndex(7, &argMember);
				if(callPtr->nargs > 7) {	// cast name or number is optional
					argCastGiven = 1;
					GetArgByIndex(8, &argCast);
				}
				//-- PL
				if (callPtr->nargs > 8) {
					GetArgByIndex(9, &argTrimWhiteSpace);
					pObj->pValueInterface->ValueToInteger(&argTrimWhiteSpace, &bTrimWhiteSpace);
				}
				//--/PL
			}

			if(argCastGiven) {
				err = pObj->pMmValue->ValueType(&argCast, &argType);
				switch(argType) {
					case kMoaMmValueType_Integer:
						err = pObj->pMmValue->ValueToInteger(&argCast, &castIndex);
						break;

					case kMoaMmValueType_String:
						err = pObj->pMmValue->ValueToString(&argCast, castName, 256);
						//err = MOVIE(GetCastIndexFromName)(MOVARG_ castName, &castIndex);
						err = pMovie->GetCastIndexFromName(castName, &castIndex); //-- PL
						
						if(err != kMoaErr_NoErr)
							retVal = -21;
						break;

					default:				// unrecognized type passed in
						retVal = -22;
				}
			}
			else							// no cast specified, assume "Internal"
				castIndex = 1;

			if(retVal != 0)
				break;						// jump out if there was an error

			//err = MOVIE(GetNthCast)(MOVARG_ castIndex, &pCast);
			err = pMovie->GetNthCast(castIndex, &pCast); //-- PL
			
			if(err != kMoaErr_NoErr) {				// invalid cast specified
				retVal = -23;
				break;
			}

			err = pObj->pMmValue->ValueType(&argMember, &argType);	// figure out if member ref is an int or string
			switch(argType) {
				case kMoaMmValueType_Integer:
					err = pObj->pMmValue->ValueToInteger(&argMember, &memberIndex);
					break;

				case kMoaMmValueType_String:
					err = pObj->pMmValue->ValueToString(&argMember, memberName, 256);
					//err = pCast->lpVtbl->GetMemberIndexFromName(pCast, memberName, &memberIndex); 
					err = pCast->GetMemberIndexFromName(memberName, &memberIndex); //-- PL
					 
					if(err != kMoaErr_NoErr) {
						//pCast->lpVtbl->Release(pCast);
						pCast->Release(); //-- PL
						
						retVal = -31;
						break;
					}
					break;

				default:							// unrecognized type passed in
					//pCast->lpVtbl->Release(pCast);
					pCast->Release(); //-- PL
					
					retVal = -32;
					break;
			}

			if(retVal != 0)
				break;						// jump out if there was an error

			err = pObj->pMmValue->StringToSymbol("bitmap", &typeSymbol);
			//if(pCast->lpVtbl->CreateCastMem(pCast, memberIndex, typeSymbol) != kMoaErr_NoErr) 
			if(pCast->CreateCastMem(memberIndex, typeSymbol) != kMoaErr_NoErr) //-- PL
			{
				//pCast->lpVtbl->Release(pCast);
				pCast->Release(); //-- PL
				
				retVal = -41;
				break;
			}
			
			lOptions = kMoaDrImgMediaOpts_ImageDepthAndPal;
			//-- PL
			if (! bTrimWhiteSpace)
				lOptions |= kMoaDrImgMediaFlags_DontTrimWhiteEdges;
			//--/PL

			pObj->pMmValue->StringToSymbol("Image", &labelSymbol);
			#ifdef MACINTOSH
				err = pObj->pMmValue->StringToSymbol("macPICT", &formatSymbol);
				err = pObj->pDrUtils->NewMediaInfo(labelSymbol, formatSymbol,
								(PMoaVoid) picHdl, lOptions,
								(PMoaVoid) NULL, &mediaInfo);
			#else
				err = pObj->pMmValue->StringToSymbol("winDIB", &formatSymbol);
				err = pObj->pDrUtils->NewMediaInfo(labelSymbol, formatSymbol,
								(PMoaVoid) hDIB, lOptions,
								(PMoaVoid) NULL, &mediaInfo);
			#endif
		
			if(err != kMoaErr_NoErr) {
				//pCast->lpVtbl->Release(pCast);
				pCast->Release(); //--PL
				
				retVal = -42;
				break;
			}

			//if(pCast->lpVtbl->GetCastMem(pCast, memberIndex, &pCastMem) != kMoaErr_NoErr) 
			if(pCast->GetCastMem(memberIndex, &pCastMem) != kMoaErr_NoErr) //--PL
			{
				//pCast->lpVtbl->Release(pCast);
				pCast->Release(); //--PL
				
				retVal = -43;
				break;
			}

			#ifdef MACINTOSH
				mediaInfo.mediaData = picHdl;
			#else
				mediaInfo.mediaData = hDIB;
			#endif
		
			//if(pCastMem->lpVtbl->SetMedia(pCastMem, &mediaInfo) != kMoaErr_NoErr) 
			if(pCastMem->SetMedia(&mediaInfo) != kMoaErr_NoErr) //-- PL
			{
				//pCastMem->lpVtbl->Release(pCastMem);
				//pCast->lpVtbl->Release(pCast);
				pCastMem->Release(); //-- PL
				pCast->Release(); //-- PL
				
				retVal = -44;
				break;
			}

			//pCastMem->lpVtbl->Release(pCastMem);
			//pCast->lpVtbl->Release(pCast);
			pCastMem->Release(); //-- PL
			pCast->Release(); //-- PL
				
			break;
			
	}

	#ifdef MACINTOSH
		KillPicture(picHdl);
	#else
		DeleteObject(hBitmap);
	//	DeleteObject(hPal);
		GlobalFree(hDIB);
	//	GlobalFree(hPal);
	#endif

	pObj->pMmValue->IntegerToValue(retVal, &callPtr->resultValue);
	
	//return(0);
	
moa_catch
moa_catch_end	
moa_try_end	
}

/*** ??? seem to be not used
void GetDirRGB(int ndx, int *r, int *g, int *b)
{
	MoaMmRGBTriple RGBColor;
	CScrnUtilXtra_IMoaDrXLingo FAR *This;

	This = globalThis;
	MmUtils_IndexToRGB((MoaMmColorIndex) ndx, &RGBColor);
	*r = RGBColor.red;
	*g = RGBColor.green;
	*b = RGBColor.blue;
}
***/

// for debug purpose
void MyAlert(char * szMsg)
{
	MoaMmValue		val = {0};
	MoaMmSymbol		sym;
	
	gMmValue->StringToSymbol("alert", &sym);
	gMmValue->StringToValue(szMsg, &val);
	gDrPlayer->CallHandler(sym, 1, &val, 0);
}

